function [ln_params] = ALL_fitLn(case_id, mu_d0, sigma_d0, rho_p)

% ALL_FITLN finds the parameters for the closest lognormal distribution 
% to the normal distribution specified by mu_d0 and sigma_d0 
%    
%   INPUTS:
%   case_id        - Test case flag:   
%                    1 -> Dissolution only
%                    2 -> Aggregation only 
%                    3 -> Simultaneous dissolution and aggregation
%   mu_d0          - Initial mean particle diameter in nm  
%   sigma_d0       - Initial standard dev. of the particle diameter in nm
%   rho_p          - NP density in g/m^3
%
%   OUTPUTS:
%   ln_params      - Parameters of the initial lognormal distribution
%                    returned by the fitting procedure

    %Convert input mean and standard deviation to radius in meters 
    mu_r = (mu_d0/2)*10^(-9); 
    sigma_r = (sigma_d0/2)*10^(-9); 

    
    %% [1] Using the same equations that we use to bin particles in the 
    %  Sectional Method, evaluate (construct) a normal distribution with  
    %  mean mu_r and standard deviation sigma_r on an arbitrary expanding 
    %  grid of particle sizes (to which we will later fit the lognormal 
    %  distribution):
    
    % Specify arbitrary lower and upper bounds (in m) for the size grid
    min_r = 1e-10;
    max_r = 2e-9*mu_d0;
    min_m = rho_p*(4/3)*pi*(  min_r  )^3;
    max_m = rho_p*(4/3)*pi*(  max_r  )^3;

    % Specify the grid expansion parameter
    if(case_id == 3)
        m_rat = 1+1e-2;
    else
        m_rat = 1+1e-3;
    end

    % Specify the number of bins
    n_bins = ceil(1+( log(max_m/min_m) / log(m_rat) ))+1;
    masses = zeros(n_bins,1);

    % Specify the size grid
    for bin_i = 1:n_bins
        masses(bin_i,1) = (m_rat^bin_i)*min_m;
    end

    % Use the geometry of a sphere to convert particle masses to radii
    radii = ((3.*masses)./(4*rho_p*pi)).^(1/3);

    % Evaluate a normal distribution for the values on the size grid
    norm_rad_pdf = normpdf(radii,mu_r,sigma_r);
    
    % Perform a coordinate transformation to convert the distribution 
    % from one in terms of radius to one in terms of mass
    fit = norm_rad_pdf./(4*rho_p*pi*(radii.^2));
    %%


    %% [2] Make an initial guess at the parameters of the lognormal 
    %  distribution.  This guess is not exactly correct because sigma_m as
    %  calculated below is only approximately correct: sigma_r cannot be
    %  converted directly to sigma_m (instead, a coordinate transformation 
    %  is needed like the one shown above)
    
    % Ballpark the mean and standard deviation of the normal particle size
    % distribution in terms of mass instead of radius
    mu_m = rho_p*(4/3)*pi*(mu_r)^3; % Mean particle radius in meters
    sigma_m = rho_p*(4/3)*pi*(mu_r+sigma_r)^3 - rho_p*(4/3)*pi*(mu_r)^3;

    % Randomly sample the normal distribution with this mu and sigma
    fit2 = normrnd(mu_m,sigma_m,10000,1); 
    
    % Fit the sample to the nearest lognormal distribution
    x0 = lognfit(fit2(fit2 > 0)); 
    %%
    
    
    % With [1] a normal distribution that we can fit our lognormal 
    % distribution to and [2] an initial guess at the values of ln_params, 
    % we can use least squares curve fitting to find the parameter values  
    % for the initial lognormal distribution that is closest in shape to 
    % the normal distribution with mean mu_d0 and standard deviation
    % sigma_d0
    fun = @(x,mplot) lognpdf(mplot,x(1),x(2));
    [x,~,~,~,~] = lsqcurvefit(fun,x0,masses,fit);

    ln_params(1) = x(1);
    ln_params(2) = x(2);

return

end