%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Given the user-specified inputs defined here, CONTROL_FILE
% (1) specifies an initial nanoparticle (NP) size distribution,
% (2) calls functions that track changes in statistics of the distribution 
%     over time during dissolution, aggregation, or both using an 
%     analytical solution (if available) and one of several numerical 
%     solutions, and
% (3) plots predicted statistics, as well as error in the numerical 
%     solution relative to the analytical solution (if available)
%
% NOTES:
% (1) Detailed methods are given as Supplementary Information
%     in Dale et al. (2016):
%     A. Dale, G. Lowry, E. Casman. Accurate and fast numerical models for 
%     tracking particle size distributions during nanoparticle aggregation 
%     and dissolution. 2016. Environmental Science: Nano.
%
% (2) The chosen size coordinate is mass.  Use of a different  
%     coordinate (e.g., radius) may require substantial modification of
%     the code.
%
% (3) The code solves for the raw fractional moment sequence 
%     k = 0, 1/3, 2/3, 1 ...  The fraction used to generate this sequence 
%     (1/3) is hard-coded throughout, rather than user-specified, because 
%     it cannot safely be altered without substantial code modification.
%
% (4) The analytical solutions only apply to initially lognormal
%     distributions.  The numerical solutions apply to any arbitrary
%     distribution.  However, if the initial distribution is not lognormal,
%     the code must be altered so that the initial moments and 
%     characteristic radius of the distribution are provided as inputs 
%     rather than calculated from the distribtion parameters at time 0.
%
%   Author: Amy L. Dale (contact: adale4552@gmail.com)
%   Code written at Carnegie Mellon University  
%   Last modified: August 4, 2016
%
% Copyright notice: We grant the reader full permission for any 
% non-commercial or educational use and/or modification of this code, 
% with proper attribution.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clearvars % Clear the workspace 

warning('off','all'); % Toggle warnings on/off

% Set default figure properties
set(0, 'DefaultAxesColor', 'White', 'DefaultAxesFontSize', 14, ...
    'DefaultTextFontSize', 14, 'DefaultTextFontName', 'Calibri', ...
    'DefaultFigureColor','w');



%% ________________________________________________________________________
%                               
%                          I. Specify inputs    
%__________________________________________________________________________


%% --------------------------------------------------------------------- %%
%%%%%%%%%%%%%%%%% Specify test case and numerical method %%%%%%%%%%%%%%%%%%
%% --------------------------------------------------------------------- %%

% Use an integer flag to specify the test case:
% case_id = 1 -> dissolution only
% case_id = 2 -> aggregation only
% case_id = 3 -> simultaneous aggregation and dissolution
case_id = 1;

% sink = 0 -> dissolution proceeds to an equilibrium ion concentration
% sink = 1 -> dissolution occurs under sink conditions, i.e. the ion 
%             concentration remains zero at all times
sink = 1;

% fusion = 0 -> particles do not fuse upon aggregation
% fusion = 1 -> particles fuse completely upon aggregation
fusion = 0;

% Specify the skew to the initial PSD (factor by which the
% scale parameter of the lognormal distribution is modified)
% Accepted (default) values are 0.25, 1 (base case), 2, and 4 as shown in
% Figure 5 of Dale et al. (2016)
skew = 1;

% Specify the numerical method to be implemented:
% 'SM' = Sectional Method
% 'DQMOM' = Direct Quadrature Method of Moments
% 'BEQMOM' = Beta Extended Quadrature Method of Moments
% 'DSMC' = Direct Simulation Monte Carlo
method = 'SM';

% Specify the number of raw fractional moments needed to solve for
% the statistics of interest, e.g.
%  num_moments = 1 -> solve up to the 0th moment (for number concentration)
%  num_moments = 2 -> " " " " 1/3rd moment (for mean particle diameter)
%  num_moments = 3 -> " " " " 2/3rd moment (for surface area concentration)
%  num_moments = 4 -> " " " " 1st moment (for mass concentration)
num_moments = 4;


%% --------------------------------------------------------------------- %%
%%%%%%%%%%%%%%%%%%%%%%%%% Check for invalid inputs %%%%%%%%%%%%%%%%%%%%%%%%
%% --------------------------------------------------------------------- %%

if(case_id~=1 && case_id~=2 && case_id~=3)
    error('"case_id" must be 1, 2, or 3');
end

if(sink~=0 && sink~=1)
    error('"sink" must be 0 or 1');
end

if(fusion~=0 && fusion~=1)
    error('"fusion" must be 0 or 1');
end

if(~strcmp(method,'SM') && ~strcmp(method,'DQMOM') && ~strcmp(method,'BEQMOM')&& ~strcmp(method,'DSMC'))
    error('"method" must be ''SM'', ''DQMOM'', ''BEQMOM'', or ''DSMC''.');
end

if(num_moments < 4 || num_moments > 4)
    fprintf(2,['WARNING: Code was designed for num_moments = 4.  SM\n' ...
          'and DSMC require num_moments >= 4 to execute.  Values\n' ...
          'greater than four may require an increase in the number\n' ...
          'of quadrature points in the QMOM methods, and may\n' ...
          'increase numerical error.\n']);
end


%% --------------------------------------------------------------------- %%
%%%%%% Parameters describing the initial particle size distribution %%%%%%% 
%%%%%% and the number of days for the test cases presented in Dale  %%%%%%%
%%%%%% et al. (2016)                                                %%%%%%%
%% --------------------------------------------------------------------- %%

mu_d0 = 50; % Initial mean particle diameter in nm  

% sigma_d0 -> Initial standard deviation of the particle diameter in nm
% N0 -> Initial particle number concentration in #/m^3
% Note: N0 was chosen (for the given value of mu_d0 and sigma_d0) 
% to give an initial mass concentration of 100 g/m^3
if(case_id == 1 || case_id == 2)
    if(mu_d0 == 50)
        sigma_d0 = 10; 
        if(skew == 1)
            N0 = 2.0750e+17;
        elseif(skew == 2) 
            N0 = 1.0187e+17;   
        elseif(skew == 4) 
            N0 = 5.9178e+15;
        elseif(skew == 0.25)
            N0 = 2.5915e+17;
        end
    elseif(mu_d0 == 100)
    	sigma_d0 = 20; 
        N0 = 2.5936e+16;
    elseif(mu_d0 == 500)
    	sigma_d0 = 100; 
        N0 = 2.0739e+14;
    elseif(mu_d0 == 15)
    	sigma_d0 = 3; 
        N0 = 7.6850e+18;
    elseif(mu_d0 == 5)
    	sigma_d0 = 1; 
        N0 = 2.0749e+20;
    end
elseif(case_id == 3)
    % In the case of simultaneous aggregation and dissolution, sigma_d0
    % must be small because the solution approach assumes monodispersity
    sigma_d0 = 2; 
    N0 = 2.7006e+17;
end


% Specify the number of days in the simulation
if(case_id == 1 && sink == 1)
    if(mu_d0 == 5)
        n_days =     0.042;    
    elseif(mu_d0 == 15)
        n_days =     0.5;      
    elseif(mu_d0 == 50)
        if(skew == 1)
            n_days =     2;
        elseif(skew == 2)
            n_days =     5;
        elseif(skew == 4)
            n_days =     36;
        elseif(skew == 0.25)
            n_days =     3;
        end
    elseif(mu_d0 == 100)
        n_days =     5; 
    elseif(mu_d0 == 500)
        n_days =     25;   
    end
elseif(case_id == 1 && sink == 0)
    if(mu_d0 == 5)
        n_days =     0.042;  
    elseif(mu_d0 == 15)
        n_days =     0.042;    
    elseif(mu_d0 == 50)
        n_days =     5*0.042;
    elseif(mu_d0 == 100)
        n_days =     8*0.042;      
    elseif(mu_d0 == 500)
        n_days =     1;    
    end
else
    n_days = 2;
end


%% --------------------------------------------------------------------- %%
%%%%%%%%%%%%%%%%%%%%%%%% Specify constants %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% --------------------------------------------------------------------- %%

rho_p = 5.61*(10^6); % NP density in g/m^3, given for ZnO
V_mol = 1.45e-5;     % molar volume in m^3/mol, given for ZnO

% Surface free energy, given for (bulk) ZnO
% from Mudunkotuwa et al. (2011)
gamma = 0.5;      

% Equilibrium solubility for bulk ZnO
% from Clever et al. (1992)
bulk_S = 4.1;        

% Surface-dependent dissolution rate (determined by calibration)
% as given in Dale et al. (2016) [Eq. S6]
k_S_pb = 4e-6;

% Power the size coordinate is raised to in Dale et al. (2016) [Eq. S6]
pow = 2/3;  

% Stoichiometric coefficient (g Me+ generated per g MexOx dissolved)
% (given for Zn and expressed in terms of the molecular weights of Zn 
% and ZnO)
g_rat=65.38/(65.38+15.9994);  

 % Empirical fitting parameter (attachment efficiency) as shown in Dale et
 % al. (2015) [Eq. S4]
alpha_agg = 0.065;

R = 8.314; % Gas constant in J/(K-mol)
kB = 1.3806488*10^(-23);   % Boltzmann constant in (m^2*kg)/(s^2*K)
visc = 0.891*10^(-3);   % Dynamic viscosity of water in kg/(m*s)
T_K = 298.15; % Temperature in Kelvin

% Prefactor for aggregation in m^3/d
% For convenience, the attachment efficiency (alpha_agg) and the prefactor
% for beta (Eq. S9) are combined into a single value
% Note the unit conversion from m3/s to m^3/d (factor of 86400)
prefac_agg = alpha_agg*((2*kB*T_K)/(3*visc))*(86400);

Df = 1.8; % Assumed fractal dimension of aggregates


%% --------------------------------------------------------------------- %%
%%%% Suggested time steps (delt) and lower and upper bounds on the mass %%%
%%%% coordinate (min_m and max_m, which is expressed in terms of radius %%%
%%%% for convenience, then converted) for test cases presented in Dale  %%%
%%%% et al. (2016)                                                      %%%
%%%%                                                                    %%%
%%%% Notes:                                                             %%%
%%%% (1) The suggested values were chosen for numerical stability and   %%% 
%%%% to maximize (to the extent possible) the accuracy and efficiency   %%%
%%%% of the test cases presented in Figure 2 of Dale et al. (2016).     %%%
%%%% Values are only suggestions and  the user should not feel          %%%
%%%% contrained by them.                                                %%%
%%%% (2) If case_id is 2 or 3, two timesteps are used in order to       %%%
%%%% speed up simulations: a small timestep needed early on             %%%
%%%% (before t_switch) to ensure numerical stability, and a larger      %%%
%%%% timestep that suffices later in the simulation (after t_switch).   %%%
%%%% (3) Empty values ([]) are specified if the value is not needed.    %%%
%% --------------------------------------------------------------------- %%

% Suggestions for the analytical solution
% Although the analytical solution is only specified for (case_id = 2) and
% (case_id = 1 and sink = 1), these bounds are also used to calculate the
% initial characteristic radius of the particles and so should be defined
% for all cases
if(case_id == 2)
    if(mu_d0 == 5)
        min_r_analyt = 1e-10;  max_r_analyt = 2000e-9; 
    else
        min_r_analyt = 1e-10;  max_r_analyt = 3000e-9; 
    end
else
    if(mu_d0 == 5)
        min_r_analyt = 1e-10; max_r_analyt = 15e-9;
    elseif(mu_d0 == 15)
        min_r_analyt = 1e-10; max_r_analyt = 50e-9;
    elseif(mu_d0 == 50)
        min_r_analyt = 1e-10; max_r_analyt = 100e-9;
        
        if(skew == 2)
            max_r_analyt = 400e-9;
        elseif(skew == 4)
            max_r_analyt = 3000e-9;
        elseif(skew == 0.25)
            max_r_analyt = 40e-9;
        end
    elseif(mu_d0 == 100)
        min_r_analyt = 1e-10; max_r_analyt = 300e-9;
    elseif(mu_d0 == 500)
        min_r_analyt = 1e-10; max_r_analyt = 1000e-9;
    end
end

if(case_id == 2 || case_id == 3)
    t_switch = (0.1/24);
else
    t_switch = [];
end

% Suggestions for the numerical solutions
if(mu_d0 == 50)
    if(case_id == 1)
        if(strcmp(method,'SM'))
            delt = 0.0005;   delt2 = [];    min_r = 1e-10; max_r = 100e-9;   
            if(skew == 2)
                max_r = 400e-9;  
            elseif(skew == 4)
                max_r = 3000e-9;
            elseif(skew == 0.25)
                delt = 0.0005;   min_r = 5e-10;  max_r = 40e-9; 
            end
        elseif(strcmp(method,'DQMOM'))
            delt = 0.001;     delt2 = [];     min_r = [];    max_r = [];
        elseif(strcmp(method,'BEQMOM'))
            delt = 0.0005;   delt2 = [];    min_r = 1e-10; max_r = 100e-9;
        elseif(strcmp(method,'DSMC'))
            delt = 0.0005;                   min_r = [];    max_r = [];  
        end
    elseif(case_id == 2)
        if(strcmp(method,'SM'))
            delt = 0.00005;  delt2 = 0.001;  min_r = 1e-10; max_r = 2000e-9;       
        elseif(strcmp(method,'DQMOM'))
            delt = 0.0001;  delt2 = 0.001;  min_r = [];    max_r = [];   
        elseif(strcmp(method,'BEQMOM'))
            delt = 0.00005;  delt2 = 0.001;  min_r = 1e-10; max_r = 2000e-9;
        elseif(strcmp(method,'DSMC'))
            delt = [];                       min_r = [];    max_r = [];  
        end
    elseif(case_id == 3 && fusion == 0)
        if(strcmp(method,'SM'))
            delt = 0.00005; delt2 = 0.001; min_r = 1e-10; max_r = 2000e-9;   %2000
        elseif(strcmp(method,'DQMOM'))
            delt = 0.000001; delt2 = 0.001;  min_r = [];    max_r = [];  
        elseif(strcmp(method,'BEQMOM'))
            % In this special case, the code is run twice and different 
            % inputs may be specified for max_r, delt, and delt2 for the first
            % and second passes
            delt = 0.001;   delt2 = 0.001;  min_r = 1e-10; max_r = 100e-9;    
            delt_pass2 = 0.00005; delt2_pass2 = 0.001; max_r_pass2 = 2000e-9;
        elseif(strcmp(method,'DSMC'))
            delt = 0.0005;                   min_r = [];    max_r = [];  
        end
    elseif(case_id == 3 && fusion == 1)
        if(strcmp(method,'SM'))
            delt = 0.00005;  delt2 = 0.001;  min_r = 1e-10; max_r = 2000e-9;   
        elseif(strcmp(method,'DQMOM'))
            delt = 0.000001; delt2 = 0.001; min_r = [];    max_r = [];  
        elseif(strcmp(method,'BEQMOM'))
            delt = 0.00005;  delt2 = 0.001;  min_r = 1e-10; max_r = 2000e-9;
        elseif(strcmp(method,'DSMC'))
            delt = 0.0001;                   min_r = [];    max_r = [];  
        end
    end
elseif(mu_d0 == 500)
    if(case_id == 1)
        if(strcmp(method,'SM'))
            delt = 0.001;   delt2 = [];    min_r = 1e-10; max_r = 1000e-9;
        elseif(strcmp(method,'DQMOM'))
            delt = 0.01;     delt2 = [];     min_r = [];    max_r = [];
        elseif(strcmp(method,'BEQMOM'))
            delt = 0.001;   delt2 = [];    min_r = 1e-10; max_r = 1000e-9;  
        elseif(strcmp(method,'DSMC'))
            delt = 0.01;                   min_r = [];    max_r = [];  
        end
    elseif(case_id == 2)
        if(strcmp(method,'SM'))
            delt = 0.001;  delt2 = 0.01;  min_r = 1e-10; max_r = 2000e-9;       
        elseif(strcmp(method,'DQMOM'))
            delt = 0.01;  delt2 = 0.01;  min_r = [];    max_r = [];   
        elseif(strcmp(method,'BEQMOM'))
            delt = 0.005;  delt2 = 0.005;  min_r = 1e-10; max_r = 2000e-9;
        elseif(strcmp(method,'DSMC'))
            delt = [];                       min_r = [];    max_r = [];  
        end
    end
elseif(mu_d0 == 100)
    if(case_id == 1)
        if(strcmp(method,'SM'))
            delt = 0.0001;   delt2 = [];    min_r = 1e-10; max_r = 300e-9;
        elseif(strcmp(method,'DQMOM'))
            delt = 0.01;     delt2 = [];     min_r = [];    max_r = [];
        elseif(strcmp(method,'BEQMOM'))
            delt = 0.001;   delt2 = [];    min_r = 1e-10; max_r = 300e-9;
        elseif(strcmp(method,'DSMC'))
            delt = 0.01;                   min_r = [];    max_r = [];  
        end
    elseif(case_id == 2)
        if(strcmp(method,'SM'))
            delt = 0.0001;  delt2 = 0.001;  min_r = 1e-10; max_r = 2000e-9;       
        elseif(strcmp(method,'DQMOM'))
            delt = 0.0001;  delt2 = 0.001;  min_r = [];    max_r = [];   
        elseif(strcmp(method,'BEQMOM'))
            delt = 0.0001;  delt2 = 0.001;  min_r = 1e-10; max_r = 2000e-9;
        elseif(strcmp(method,'DSMC'))
            delt = [];                       min_r = [];    max_r = [];  
        end
    end
elseif(mu_d0 == 15)
    if(case_id == 1)
        if(strcmp(method,'SM'))
            delt = 0.0001;   delt2 = [];    min_r = 1e-10; max_r = 50e-9;
        elseif(strcmp(method,'DQMOM'))
            delt = 0.0001;     delt2 = [];     min_r = [];    max_r = [];
        elseif(strcmp(method,'BEQMOM'))
            delt = 0.0005;   delt2 = [];    min_r = 1e-10; max_r = 50e-9;  
        elseif(strcmp(method,'DSMC'))
            delt = 0.00005;                   min_r = [];    max_r = [];  
        end
    elseif(case_id == 2)
        if(strcmp(method,'SM'))
            delt = 0.000001;  delt2 = 0.001;  min_r = 1e-10; max_r = 2000e-9;       
        elseif(strcmp(method,'DQMOM'))
            delt = 0.000001;  delt2 = 0.001;  min_r = [];    max_r = [];   
        elseif(strcmp(method,'BEQMOM'))
            delt = 0.000001;  delt2 = 0.001;  min_r = 1e-10; max_r = 2000e-9;
        elseif(strcmp(method,'DSMC'))
            delt = [];                       min_r = [];    max_r = [];  
        end
    end
elseif(mu_d0 == 5)
    if(case_id == 1)
        if(strcmp(method,'SM'))
            delt = 0.000005;   delt2 = [];    min_r = 1e-10; max_r = 100e-9;
        elseif(strcmp(method,'DQMOM'))
            delt = 0.000005;     delt2 = [];     min_r = [];    max_r = [];
        elseif(strcmp(method,'BEQMOM'))
            delt = 0.00005;   delt2 = [];    min_r = 1e-10; max_r = 20e-9;  
        elseif(strcmp(method,'DSMC'))
            delt = 0.00005;                   min_r = [];    max_r = [];  
        end
    elseif(case_id == 2)
        if(strcmp(method,'SM'))
            delt = 0.00000005;  delt2 = 0.001;  min_r = 1e-10; max_r = 2000e-9;         
        elseif(strcmp(method,'DQMOM'))
            delt = 0.0000001;  delt2 = 0.001;  min_r = [];    max_r = [];   
        elseif(strcmp(method,'BEQMOM'))
            delt = 0.00000005;  delt2 = 0.001;  min_r = 1e-10; max_r = 2000e-9;   
        elseif(strcmp(method,'DSMC'))
            delt = [];                       min_r = [];    max_r = [];  
        end
    end
end

% Calculate lower and upper bounds on the particle mass from specified
% lower and upper bounds on the particle radius (assuming a spherical 
% particle)
min_m_analyt = rho_p*(4/3)*pi*(min_r_analyt)^3;
max_m_analyt = rho_p*(4/3)*pi*(max_r_analyt)^3;

min_m = rho_p*(4/3)*pi*(min_r)^3;
max_m = rho_p*(4/3)*pi*(max_r)^3;

if(strcmp(method,'BEQMOM') && (case_id == 3 && fusion == 0))
    max_m_pass2 = rho_p*(4/3)*pi*( max_r_pass2 )^3;
end

% Create a vector of time steps, tvec (of length n_t) at which to solve.
% DSMC builds the vector of time steps dynamically, so tvec is not
% pre-specified.
if(~strcmp(method,'DSMC'))
    if(case_id == 1)
        n_t = ceil((n_days/delt))+1;

        tvec = zeros(1,n_t);
        for t_index = 1:n_t
            tvec(1,t_index) = (t_index-1)*delt;
        end
    elseif(case_id == 2 || case_id == 3)
        tvec = 0;
        while tvec(end) < t_switch
            tvec(end+1)=tvec(end)+delt;
        end
        t_switch_index = length(tvec);
        while tvec(end) < n_days
            tvec(end+1)=tvec(end)+delt2;
        end
    end
end

% Calculate tvec for the second pass in the simultaneous dissolution and
% aggregation case without fusion
if(case_id == 3 && fusion == 0 && strcmp(method,'BEQMOM'))
    tvec_pass2 = 0;
    while tvec_pass2(end) < t_switch
        tvec_pass2(end+1)=tvec_pass2(end)+delt_pass2;
    end
    t_switch_index = length(tvec_pass2);
    while tvec_pass2(end) < n_days
        tvec_pass2(end+1)=tvec_pass2(end)+delt2_pass2;
    end
end


%% ________________________________________________________________________
%                               
%             II. EXECUTE ANALYTICAL AND NUMERICAL SOLUTIONS
%__________________________________________________________________________
    
% If case_id is 3 and fusion is 0, a "dissolution only" pass is run
% prior to the "simultaneous dissolution and aggregation" pass.  
% case3_pass tells the script which pass to execute (1 -> dissolution
% only, 2 -> simultaneous dissolution and aggregation).
case3_pass = 1;

% Find the parameters for the lognormal distribution that most closely
% resembles the normal distribution specified by mu_d0 and sigma_d0.  
% Note: The user may ignore mu_d0 and sigma_d0 and instead specify 
% their own lognormal distribution with parameters ln_params(1) and 
% ln_params(2) here
[ln_params] =  ALL_fitLn(case_id, mu_d0, sigma_d0, rho_p);

% skew the initial distribution by the factor 'skew' (specified above)
ln_params(2) = skew*ln_params(2);

% Calculate the characteristic radius at the first time step assuming an
% initially lognormal distribution with parameters specified by ln_params
% The formula for the pdf of a lognormal distribution is Dale et al. (2016)
% [Eq. S46]
% The characteristic (surface-weighted geometric mean) radius is [Eq. S3]
pdf0 = @(m) N0.*((1./(ln_params(2)*sqrt(2*pi).*m)).*exp(-(( log(m)-ln_params(1)).^2) ./ (2*(ln_params(2)^2)) ));
char_rad0_numer = @(m) ((m.^(2/3)).*log((((6/(rho_p*pi))^(1/3)))*m.^(1/3))).*pdf0(m);
char_rad0_denom = @(m)  (m.^(2/3)).* pdf0(m);
char_rad0 =   exp(quadgk(char_rad0_numer, min_m_analyt, max_m_analyt)/quadgk(char_rad0_denom, min_m_analyt, max_m_analyt))/2;

% Use char_rad0 to calculate the equilibrium ion concentration (solubility) 
% in g/m^3.  The formula for S is the Ostwald-Freundlich relation [Eq. S2]
S = bulk_S*exp((4*gamma*V_mol)/(R*T_K*(2*char_rad0)));


% Execute the numerical solution ("pass one" if case_id == 3 and fusion = 0)
if(strcmp(method,'SM'))
    [stats, Sauter_num, time1] =  method_SM(case_id, sink, fusion, case3_pass, num_moments, delt, delt2, t_switch, tvec, N0, S, rho_p, g_rat, k_S_pb, pow, prefac_agg, Df, ln_params, char_rad0, min_m, max_m, [], mu_d0);
elseif(strcmp(method,'DQMOM'))
    [stats, Sauter_num, time1] =  method_DQMOM(case_id, sink, fusion, case3_pass, delt, delt2, t_switch, tvec, N0, S, rho_p, g_rat, k_S_pb, pow, prefac_agg, Df, ln_params, char_rad0, []);
elseif(strcmp(method,'BEQMOM'))
    [stats, tvec, Sauter_num, time1] =  method_BEQMOM(case_id, sink, fusion, case3_pass, num_moments, delt, delt2, t_switch, tvec, [], N0, S, rho_p, g_rat, k_S_pb, pow, prefac_agg, Df, ln_params, char_rad0, min_m, max_m, []);
elseif(strcmp(method,'DSMC'))
    [stats, tvec, Sauter_num, time1] =  method_DSMC(case_id, sink, fusion, case3_pass, num_moments, n_days, delt, [], N0, S, rho_p, g_rat, k_S_pb, pow, prefac_agg, Df, ln_params, char_rad0, []);
end



% If case_id = 3 and fusion = 0, execute "pass two" of the numerical 
% solution:
case3_pass = 2;

% time2 will hold the simulation runtime for the second pass 
% (defaults to zero if no second pass is needed)
time2 = 0; 

if(case_id == 3 && fusion == 0)
    
    if(strcmp(method,'SM'))
        [stats, Sauter_num, time2] =  method_SM(case_id, sink, fusion, case3_pass, num_moments, delt, delt2, t_switch, tvec, N0, S, rho_p, g_rat, k_S_pb, pow, prefac_agg, Df, ln_params, char_rad0, min_m, max_m, Sauter_num, mu_d0);
    elseif(strcmp(method,'DQMOM'))
        [stats, Sauter_num, time2] =  method_DQMOM(case_id, sink, fusion, case3_pass, delt, delt2, t_switch, tvec, N0, S, rho_p, g_rat, k_S_pb, pow, prefac_agg, Df, ln_params, char_rad0, Sauter_num);
    elseif(strcmp(method,'BEQMOM'))
        [stats, tvec, Sauter_num, time2] =  method_BEQMOM(case_id, sink, fusion, case3_pass, num_moments, delt_pass2, delt2_pass2, t_switch, tvec_pass2, tvec, N0, S, rho_p, g_rat, k_S_pb, pow, prefac_agg, Df, ln_params, char_rad0, min_m, max_m_pass2, Sauter_num);
    elseif(strcmp(method,'DSMC'))
        [stats, tvec, Sauter_num, time2] =  method_DSMC(case_id, sink, fusion, case3_pass, num_moments, n_days, delt, tvec, N0, S, rho_p, g_rat, k_S_pb, pow, prefac_agg, Df, ln_params, char_rad0, Sauter_num);
    end
end

% The simulation runtime is the runtime for the first and(if applicable) 
% second passes
time = time1+time2;

% Find the analytical solution, if available for the given test case
if((case_id == 1 && sink == 1) || case_id == 2) 
    [stats_analyt] =  ANALYT(case_id, sink, num_moments, tvec, N0, S, rho_p, k_S_pb, pow, prefac_agg, Df, ln_params, char_rad0, min_m_analyt, max_m_analyt);
end



%% ________________________________________________________________________
%                               
%                         III.  VISUALIZE OUTPUT
%__________________________________________________________________________

disp(['Runtime for numerical solution: ' num2str(time) ' seconds']);

% When "hold on" is uncommented, results from successive trial runs 
% (e.g., different numerical methods applied to a given test case) are 
% overlain on the same plot
hold on;

% Plot the statistics of interest calculated from the raw fractional 
% moments
for k = 1:num_moments

    subplot(3,num_moments,[k,k+num_moments]);
    
    % If an analytical solution is available, plot it
    if((case_id == 1 && sink == 1) || case_id == 2) 
        plot(tvec*24, stats_analyt(k,:),'k', 'LineWidth',4); hold on; 
    end
    
    % Specify line type for plotting
    if(strcmp(method,'SM'))
        line_type = '-.b';
        width = 1.5;   
    elseif(strcmp(method,'DQMOM'))
        line_type = '--m';
        width = 1.5;  
    elseif(strcmp(method,'BEQMOM'))
        line_type = ':r';
        width = 2; 
    elseif(strcmp(method,'DSMC'))
        line_type = '-g';
        width = 1.25;     
    end
    
    plot(tvec*24, stats(k,:), line_type, 'LineWidth', width); hold on;   
        
    title({['m' num2str((k-1)/3)]}, 'FontSize', 12)
    xlim([0,n_days*24])
    
    % If an analytical solution is available, the x-axis will be labeled
    % below the row of subplots showing error in the numerical solutions.
    % Otherwise, label it here.
    if(~((case_id == 1 && sink == 1) || case_id == 2))
        xlabel('Time (d)', 'FontSize', 14)
    end

    if(k==1)
        ylabel('Value Calculated from Moment', 'FontSize', 14);
    end

end

        
% If an analytical solution is available for comparison, calculate and plot
% the error associated with the numerical solution relative to the
% analytical solution
% To prevent asymptotic behavior in those parameters which trend to zero
% during the simulation, error is calculated relative to the initial value
% rather than the value at time t
%
if((case_id == 1 && sink == 1) || case_id == 2)

    for k = (2*num_moments+1):3*num_moments     
        subplot(3,num_moments,k);   

        if(case_id == 1 || strcmp(method,'DSMC'))
            error(1,:) = ((stats(k-2*num_moments,:)-stats_analyt(k-2*num_moments,:))/stats_analyt(k-2*num_moments,1))*100;
        else
            error(1,:) = ((stats(k-2*num_moments,t_switch_index:length(tvec))-stats_analyt(k-2*num_moments,t_switch_index:length(tvec)))/stats_analyt(k-2*num_moments,1))*100;
        end
               
        if(strcmp(method,'SM'))
            line_type = '-.b';
            width = 1.5;   
        elseif(strcmp(method,'DQMOM'))
            line_type = '--m';
            width = 1.5;  
        elseif(strcmp(method,'BEQMOM'))
            line_type = ':r';
            width = 2; 
        elseif(strcmp(method,'DSMC'))
            line_type = '-g';
            width = 1.25;     
        end

        if(case_id == 1 || strcmp(method,'DSMC'))
            plot(tvec, error, line_type, 'LineWidth', width); hold on; 
        else
            plot(tvec(t_switch_index:length(tvec)), error, line_type, 'LineWidth', width); hold on; 
        end
       
        xlim([0,n_days])
        xlabel('Time (d)', 'FontSize', 14)

        if(k==9)
            ylabel('% Error', 'FontSize', 14)
        end

    end
end

%Save workspace locally with a name that specifies the test case
str_id = {[method '_case' num2str(case_id) '_sink' num2str(sink) '_fus' num2str(fusion)]};
save(char(str_id));


% LITERATURE CITED
% For a complete list of references, see Dale et al. (2016)

% Clever, H. L., M. E. Derrick, and S. A. Johnson (1992), The Solubility of 
% Some Sparingly Soluble Salts of Zinc and Cadmium in Water and in Aqueous 
% Electrolyte Solutions, J. Phys. Chem. Ref. Data, 21(5), 941-966.

% Mudunkotuwa, I. A., T. Rupasinghe, C.-M. Wu, and V. H. Grassian (2011), 
% Dissolution of ZnO nanoparticles at circumneutral pH: a study of size 
% effects in the presence and absence of citric acid, Langmuir, 28(1), 
% 396-403.


