function [stats_frac, tvec, char_rad, time] = method_BEQMOM(case_id, sink, ...
    fusion, case3_pass, num_moments, delt, delt2, t_switch, tvec, ...
    tvec_pre, N0, S, rho_p, g_rat, k_S_pb, pow, prefac_agg, Df, ...
    ln_params, char_rad0, min_m, max_m, char_rad)

% METHOD_BEQMOM tracks changes in statistics of the distribution over time 
% during dissolution, aggregation, or both using the Beta Extended 
% Quadrature Method of Moments
%    
%   INPUTS:
%   case_id        - Test case flag:   
%                    1 -> Dissolution only
%                    2 -> Aggregation only 
%                    3 -> Simultaneous dissolution and aggregation
%   sink           - Sink flag:        
%                    0 -> Dissolution to equilibrium ion concentration
%                    1 -> Dissolution under sink conditions (no ion)
%   fusion         - Fusion flag:      
%                    0 -> Particles do not fuse upon aggregation
%                    1 -> Particles fuse completely upon aggregation
%   case3_pass     - Case 3 pass flag: 
%                    1 -> First pass (dissolution only)
%                    2 -> Second pass (dissolution & aggregation)
%
%   num_moments    - Number of raw fractional moments for which to solve
%   delt           - Time step prior to t_switch in days (case_id = 2 or 3)
%                    or for the entire simulation (case_id = 1)
%   delt2          - Time step after t_switch in days (case_id = 2 or 3)
%   t_switch       - Time in days at which the time step switches from delt 
%                    to delt2 (case_id = 2 or 3)
%   tvec           - Vector of the times in days at which to solve the
%                    statistics of interest for the current run
%   tvec_pre       - Vector of the simulation time at every time step in
%                    days for the first case 3 pass, if on the second
%
%   N0             - Initial particle number concentration in #/m^3
%   S              - Equilibrium ion concentration (solubility) in g/m^3
%   rho_p          - NP density in g/m^3
%   g_rat          - Stoichiometric coefficient for dissolution: g Me+
%                    generated per g MexOx dissolved
%   k_S_pb         - Dissolution rate
%   pow            - Power that the size coordinate is raised to in the
%                    rate law for dissolution
%   prefac_agg     - Prefactor for the aggregation rate
%   Df             - Assumed fractal dimension of aggregates
%   ln_params      - Vector containing the parameters of the initial
%                    lognormal nanoparticle size distribution: [mu, sigma]
%   min_m          - Lower bound of the size domain in m
%   max_m          - Upper bound of the size domain in m
%
%   char_rad       - Vector of the characteristic radius of primary 
%                    particles at every time step in m 
%                    (calculated and returned during the first pass of 
%                    case 3 and read in as input for the second pass)
%
%   OUTPUTS:
%   stats_frac     - Matrix of statistics of the distribution as a function
%                    of time (calculated from the raw fractional moments) 
%                    to be returned to CONTROL_FILE.m for visualization and 
%                    error analysis
%   char_rad       - Defined in INPUTS
%   time           - Simulation runtime in s

    %---------------------------------------------------------------------%
    %%%%%%%%%%%%%%% MODEL RESOLUTION (USER-SPECIFIED INPUT) %%%%%%%%%%%%%%%
    %---------------------------------------------------------------------%
 
    if(case_id == 1)
        pts = 2; % Number of primary quadrature points   
        pts2 = 20; % Number of secondary quadrature points  
    elseif(case_id == 2)
        pts = 2; 
        pts2 = 20; 
    elseif(case_id == 3)
        pts = 1; 
        pts2 = 50;    
    end 
    
    % Arbitrary upper bound on the spread (sigma) around each primary
    % quadrature point.  This can also be calculated from the Hankel
    % determinants as described in Section S1.5.c in Dale et al. (2016)
    sig_max = 1; 
    
    if(pts > 4)
        error('Code does not handle more than 4 quadrature points for EQMOM');
    end
    
    %---------------------------------------------------------------------%
    %%%%%%%%%%%%%%% PARAMETER DEFINITION AND INITIALIZATION %%%%%%%%%%%%%%%
    %---------------------------------------------------------------------%    
    
    % Number of time steps in the simulation
    n_t = length(tvec);
    
    % Number of lower-order moments that can be solved for exactly with pts
    % quadrature points
    k_max = 2*pts - 1; 

    % Holds the raw integer moments 
    stats = zeros(k_max+2,n_t);
    
    % Holds the raw integer moments that have been transformed to fall on
    % the domain [0,1]
    stats_trans = zeros(k_max+2,n_t);
    
    % Initially holds the raw fractional moments, but is later converted
    % to the desired statistics of the distribution (calculated from the 
    % raw moments) that are returned by the function
    stats_frac = zeros(num_moments,n_t);
    
    % Holds the ion mass concentration in g/m^3
    mass_conc_ion = zeros(1,n_t);
    
    % char_rad --> Holds the characteristic (surface-weighted geometric
    % mean) radius at every time step
    % If case3_pass == 2, char_rad has been passed to the function from 
    % the first pass and should not be reset.
    if(case_id == 1 || (case_id == 3 && case3_pass == 1))
        char_rad = zeros(1,n_t); 
        char_rad(1,1) = char_rad0;
    end
    
    % Holds the geometric mean number of primary particles in an aggregate 
    % (if applicable)
    mean_num_per = zeros(1,n_t); 

    % Holds the abscissas calculated in [0,1] space at time t
    x_trans_vec = zeros(pts,n_t);
    
    % Holds the weights (which are the same in [min_m, max_m] space or 
    % [0,1] space) at time t
    w_vec = zeros(pts,n_t); 
    
    % Holds the standard deviation around the primary quadrature points 
    % at time t
    sigma_vec = zeros(1,n_t); 
    
    % When the initial distribution is lognormal, the initial moments of
    % the distribution can be calculated exactly from the parameters of the
    % distribution (Eq. S48 of Dale et al. (2016))
    for k=1:k_max+2
        stats(k,1) = N0*exp(  (k-1)*ln_params(1)  + (((k-1)^2*(ln_params(2)^2))/2)   );
        stats_frac(k,1) =  N0*exp(  ((k-1)/3)*ln_params(1)  + ((((k-1)/3)^2*(ln_params(2)^2))/2)   );
    end
    
    % For BEQMOM, in the case of simultaneous aggregation and dissolution, 
    % the initial "dissolution only" pass used to calculate changes in the 
    % characteristic radius over time can have a different time step than
    % the final 'dissolution and aggregation' pass (this was done to 
    % improve model performance).  In this case, interpolate values of the 
    % characteristic radius between time steps.
    if(case_id == 3 && case3_pass == 2)
        char_rad_interp = zeros(1,n_t);
        
        % For each time step (in the second pass) ... 
        for t_index = 1:n_t-1
            
            % Find the index for the closest time step in the first pass
            sub_vec = tvec_pre-tvec(t_index+1);
            [~, min_ind]=min(abs(sub_vec));

            % If the times are the same for both passes or we are at the 
            % boundary of tvec_pre ...
            if(sub_vec(min_ind)==0 || (sub_vec(min_ind) > 0 && min_ind == 1) || (sub_vec(min_ind) < 0 && min_ind >= length(tvec_pre)-1)  )
                char_rad_interp(1,t_index) = char_rad(min_ind); 
                
            % If the closest time step from the first pass is larger than
            % the time step for the second pass, interpolate between that
            % time step and its next smallest neighbor
            elseif(sub_vec(min_ind) > 0)
                char_rad_interp(1,t_index) = char_rad(min_ind-1) + ((tvec(t_index+1)-tvec_pre(min_ind-1))*(char_rad(min_ind)-char_rad(min_ind-1)))/(tvec_pre(min_ind)-tvec_pre(min_ind-1));
                
            % If the closest time from the first pass is smaller than
            % the time step for the second pass, interpolate between that 
            % time step and its next largest neighbor
            elseif(sub_vec(min_ind) < 0)
                char_rad_interp(1,t_index) = char_rad(min_ind) + ((tvec(t_index+1)-tvec_pre(min_ind))*(char_rad(min_ind+1)-char_rad(min_ind)))/(tvec_pre(min_ind+1)-tvec_pre(min_ind));
            end
        end
        
        % Calculate the characteristic particle mass
        % (from the geometry of a sphere)
        char_mass_interp(1,:) = rho_p*(4/3)*pi*(char_rad_interp(1,:)).^3; 
        
        % Calculate the surface area concentration (m^2/m^3)
        % (from the geometry of a sphere)
        char_sa_interp(1,:) = 4*pi*(  char_rad_interp(1,:)  ).^2; 
    end
    
    %---------------------------------------------------------------------%
    %%%%%%%%%%%%%%%%%%%%%%%%%% NUMERICAL SOLUTION %%%%%%%%%%%%%%%%%%%%%%%%%
    %---------------------------------------------------------------------%
    
    tic % Start timer

    % For every time step ...
    for t_index = 1:n_t-1

        % Grab the time in days for the current time step
        t = tvec(t_index);
        
        % Grab the appropriate time step (before or after t_switch)
        if((case_id == 2 || case_id == 3) && t >= t_switch)
            delt = delt2;
        end

        % Print time at which we are solving in days
        disp(t);

        % Transform stats to [0,1] space from [min,max] space
        % Forumula is from Section 1.3.5 on p.13 of Dette and Studden
        % (1997)
        for p_ind = 1:2*pts+1
            for j = 0:(p_ind-1)
                stats_trans(p_ind,t_index) = stats_trans(p_ind,t_index) + (factorial(p_ind-1)/(factorial(j)*factorial(p_ind-1-j)))  *  (-min_m).^(p_ind-1-j)   *stats(p_ind,t_index);
            end

            stats_trans(p_ind,t_index) = (1/((max_m-min_m).^(p_ind-1)))*stats_trans(p_ind,t_index);
        end

        % Break stats_trans into subsets that will be passed to
        % a function below
        m_2Nminus = stats_trans(1:2*pts,t_index);
        m_2n = stats_trans(2*pts+1,t_index);
        
        %% Begin the non-linear search procedure to find the standard 
        %  deviation around each primary quadrature point, sigma_b:
        
        % J(sigma_b) is defined as in Yuan et al. (2012) 
        % {Eq. S39 in Dale et al. (2016))
        % J should equal zero at the end of the search procedure
        
        % The starting guess for sigma_b (standard deviation around each
        % quadrature point) is its theoretical minimum of zero.  A non-zero
        % starting guess may work better in practice, since this makes it
        % easier to search within the bounds of J space.  However, the
        % search algorithm below may need to be altered in this case, since 
        % only J(0) is guaranteed to be either positive or equal to zero
        % as assumed by the algorithm       
        sigma_b = 0;
        
        % Find the starting guess for J(sigma_b) and the weights and 
        % abscissas (w and x, respectively) suggested by sigma_b
        % Save all values of sigma_b and associated values of J that are
        % attempted during the search procedure
        [J,w,x,~] = BEQMOM_calcJ(pts, sigma_b, m_2Nminus, m_2n);
        tried_sigs = sigma_b; tried_Js = J;

        %By definition, for the first iteraration, J is either positive or
        %exactly equal to zero
        if(J > 0)
            % Apply the secant method until J(sigma_b) is less than zero
            
            % *********************ATTRIBUTION***************************
            % The code below for the secant method is reproduced with small
            % modifications from code published by Young and Mohlenkamp 
            % (2014)
            
            % Define starting guesses (outer bounds on sigma_b) for the 
            % secant method
            sig0 = sigma_b;
            [J0,~,~,~] = BEQMOM_calcJ(pts, sig0, m_2Nminus, m_2n);
            
            sig1 = sig_max;
            [J1,~,~,~] = BEQMOM_calcJ(pts, sig1, m_2Nminus, m_2n);

            if(J0 == J1)
                disp('Bad starting guess: values are the same.')
            end

            tried_sigs(end+1) = sig0; tried_Js(end+1) = J0; 

            while ( (J0 > 0 && J1 > 0) || (J0 < 0 && J1 < 0) )
                
                % secant formula
                signy = sig1 - (sig1 - sig0 )* J1 /( J1 - J0 ); 

                [Jn,~,~,~] = BEQMOM_calcJ(pts, signy, m_2Nminus, m_2n);
                tried_sigs(end+1) = signy; tried_Js(end+1) = Jn; 

                % Move numbers to get ready for the next step
                sig0 = sig1;
                J0 = J1;
                sig1 = signy;
                J1 = Jn;
            end
            
            % Once we find a value of sigma_b for which J(sigma_b) is less
            % than zero (such that we now have values of J(sigma_b) that 
            % straddle the y-axis), we use Ridder's method to find its root
            
            % *********************ATTRIBUTION***************************
            % The code below for Ridder's method is reproduced with small
            % modifications from code published by M. Kelly (2015)
            
            % Define lower and upper bounds on sigma_b and J(sigma_b)
            xLow = sig1; xUpp = sig0;
            fLow = J1; fUpp = J0; 

            if fLow*fUpp > 0.0
                error('Function has same sign at both endpoints.')
            end            

            maxIter = 50;
            for i=1:maxIter
                xMid = (xLow+xUpp)/2;

                [fMid,~,~,~] = BEQMOM_calcJ(pts, xMid, m_2Nminus, m_2n); 
                tried_sigs(end+1) = xMid; tried_Js(end+1) = fMid; 

                s = sqrt(fMid*fMid - fLow*fUpp);
                if s==0.0, break; end
                xTmp = (xMid-xLow)*fMid/s;
                if fLow >= fUpp
                    xNew = xMid + xTmp;
                else
                    xNew = xMid - xTmp;
                end
                xZero = xNew;

                [fNew,~,~,~] = BEQMOM_calcJ(pts, xZero, m_2Nminus, m_2n); 

                tried_sigs(end+1) = xZero; tried_Js(end+1) = fNew; 

                %use eps (2^(-52)) as the chosen error tolerance  
                if abs(fNew)<eps, break; end   

                if sign(fMid) ~= sign(fNew)
                    xLow = xMid;
                    fLow = fMid;
                    xUpp = xZero;
                    fUpp = fNew;
                elseif sign(fLow) ~= sign(fNew)
                    xUpp = xZero;
                    fUpp = fNew;
                elseif sign(fUpp) ~= sign(fNew)
                    xLow = xZero;
                    fLow = fNew;
                else
                    error('Ridder''s Method failed!');
                end

            end

            % Find J(sigma_b) for the best estimate of sigma_b output by
            % the secant method and Ridder's method
            sigma_b = xZero;
            [J,w,x,use_prev_flag] = BEQMOM_calcJ(pts, sigma_b, m_2Nminus, m_2n); 
            tried_sigs(end+1) = sigma_b; tried_Js(end+1) = J; 

            % The chosen value of sigma_b may not be valid.  If not, find
            % the next smallest value of J solved for during the search
            % procedure and test the validity of the associated value of 
            % sigma_b.  Repeat until a valid value of sigma_b is found or 
            % there are no more values to try (in which case the algorithm 
            % fails)
            
            % The index for the current best guess at the value of sigma_b
            % is the element we just added to the array
            min_ind = length(tried_sigs);

            counter = 0; % count the number of failed attempts
            % use_prev_flag is an integer flag returned by BEQMOM_calcJ() 
            % that indicates whether sigma_b is valid (use_prev_flag = 0) 
            % or invalid (use_prev_flag = 1) based on the realizability of
            % its associated transformed star moment set (as calculated 
            % from its Hankel determinants)
            while(use_prev_flag == 1 && counter < length(tried_sigs))
                % If the current attempt is invalid, find the next smallest
                % value of J
                tried_sigs(min_ind) = Inf; 
                tried_Js(min_ind) = Inf; 
                [~, min_ind] = min(abs(tried_Js));

                %try again
                sigma_b = tried_sigs(min_ind);
                [~,w,x,use_prev_flag] = BEQMOM_calcJ(pts, sigma_b, m_2Nminus, m_2n); 

                counter = counter + 1;
            end

            if(counter == length(tried_sigs))
                error('Tried every possibility and could not find a match')
            end
            
        elseif(J < 0)
            error('Impossible value!  J(sigma_b) in BEQMOM cannot be negative on the first iteration.');
        end
        
        %% End the non-linear search procedure to find the standard 
        %  deviation around each primary quadrature point, sigma_b
        
        
        % Find the matrices of secondary abscissas (x_ab) and weights 
        % (w_ab) associated with the chosen value of sigma_b
        [ x_ab, w_ab ] = BEQMOM_2ndQuadrature( pts, pts2, sigma_b, x );

        % A second coordinate transformation is required to shift back from 
        % the [-1,1] interval of LaGuerre polynomials to the [0,1] 
        % intervals of beta distributions
        x_ab = (x_ab+1)/2;

        % Calculate rho_ab as defined in Eq. S43 of Dale et al. (2016)
        rho_ab = zeros(pts,pts2);
        for ind = 1:pts
            for ind2 = 1:pts2
                rho_ab(ind,ind2) = w(ind).*w_ab(ind,ind2);
            end
        end
        
        % Save predicted abscissas (in [0,1] space), weights, and sigma_b 
        % values to use later in reconstructing the distributon at each
        % time step
        x_trans_vec(:,t_index+1) = x;
        w_vec(:,t_index+1) = w;
        sigma_vec(1,t_index+1) = sigma_b;
        
        % Transform abscissas back into [min,max] space from [0,1] space
        % (Weights stay the same)
        x_ab = x_ab.*(max_m - min_m) + min_m;

        % If dissolution occurs ...
        if(case_id == 1 || case_id == 3)
           
            % The flux (the change in the number concentration, or 0th  
            % moment, over time) is approximately the area under the pdf 
            % between the lower size boundary and the amount that the    
            % abscissas are expected to shift to the left during the time
            % step
            
            % Use the dissolution rate to determine the portion of the size 
            % domain over which the distribution is expected to shift
            xmin = min_m;
            xmin_plus = xmin + delt*k_S_pb*(mass_conc_ion(1,t_index)-S)*(xmin).^pow;
            
            % The pdf at time t is approximated with Eq. S34 of Dale et al.
            % (2016)
            flux = 0;
            for i = 1:pts
                pdf = @(epsilon) w(i)*(1/(max_m-min_m))*betapdf((epsilon-min_m)/(max_m-min_m),x_trans_vec(i,t_index+1)./sigma_b,(1-x_trans_vec(i,t_index+1))./sigma_b); 
                
                % use a safety factor of 1.0001 to prevent errors thrown by
                % trying to evaluate the function exactly at its lower
                % boundary
                flux = flux + quadgk(pdf,xmin*1.0001,xmin_plus); 
            end
            
            % After the flux is approximated, advect the abscissas towards
            % the lower size boundary according to their dissolution rate.
            for ind2 = 1:pts
                for ind = 1:pts2
                    x_ab(ind2,ind) = x_ab(ind2,ind) + delt*k_S_pb*(mass_conc_ion(1,t_index)-S)*(x_ab(ind2,ind)).^pow;  

                    % If any abscissas drop below the lower size boundary,
                    % set their corresponding weights to zero (particles
                    % have fully dissolved)
                    if(x_ab(ind2,ind) < min_m)
                        rho_ab(ind2,ind) = 0;
                    end

                end
            end
                   
            % Evolve the moments of the distribution using the dual
            % quadrature approximation for dissolution 
            % (Eq. S45 in Dale et al. (2016))
            for k = 1:(2*pts+1)

                delta = 0;
                for ind2 = 1:pts
                    for ind = 1:pts2
                        if(case_id == 3 && case3_pass == 2)
                            delta = delta +     (x_ab(ind2,ind)/char_mass_interp(1,t_index))*rho_ab(ind2,ind)*(char_mass_interp(1,t_index).^pow).*(x_ab(ind2,ind).^((k-1)-1))  ;
                        else
                            delta = delta +     rho_ab(ind2,ind)*(x_ab(ind2,ind).^pow).*(x_ab(ind2,ind).^((k-1)-1))  ;
                        end
                    end
                end

                stats(k,t_index+1) =  stats(k,t_index) + delt*(k-1)*k_S_pb*(mass_conc_ion(1,t_index)-S).*delta;

                if(k==1)
                    stats(k,t_index+1) = stats(k,t_index+1)+flux;
                end

            end
        end

        % If aggregation occurs ...
        if(case_id == 2 || (case_id == 3 && case3_pass == 2) || (case_id == 3 && fusion == 1))
            
            % Evolve the moments of the distribution using the dual
            % quadrature approximation for aggregation (Eq. S44 in 
            % Dale et al. (2016))
             for k = 1:(2*pts+1)
                delta_agg = 0;

                for a1 = 1:pts
                    for a2 = 1:pts
                    	for b1 = 1:pts2
                        	for b2 = 1:pts2
                            	delta_agg = delta_agg + prefac_agg*( ((x_ab(a1,b1)^(1/Df) + x_ab(a2,b2)^(1/Df))^2) / ((x_ab(a1,b1)*x_ab(a2,b2))^(1/Df)) )*rho_ab(a1,b1)*rho_ab(a2,b2)*(   (x_ab(a1,b1)+x_ab(a2,b2)).^(k-1) - x_ab(a1,b1)^(k-1) -  x_ab(a2,b2)^(k-1) );
                        	end                        
                    	end            
                    end
                end

                if(case_id == 3)
                    stats(k,t_index+1) =  stats(k,t_index+1) + delt*(0.5*delta_agg);
                else
                    stats(k,t_index+1) =  stats(k,t_index) + delt*(0.5*delta_agg);
                end
            end
        end

        % Calculate changes in the ion concentration during the time step
        % due to dissolution
        if((case_id == 1 || case_id == 3) && sink == 0)
           mass_conc_ion(1,t_index+1) = mass_conc_ion(1,t_index)+g_rat*(stats(2,t_index)-stats(2,t_index+1));  
        end
        
        % Break condition:
        % Numerical error can arise when the mass concentration, 
        % stats(2,t_index+1), gets too low, so prevent this by ending the
        % simulation early if necessary
        if(((case_id == 1) ||(case_id == 3 && case3_pass == 2) || (case_id == 3 && fusion == 1)) && stats(2,t_index+1) < 1e-3)
            break;
        end

    end    % End time loop 

    time = toc; % End timer
    
    %---------------------------------------------------------------------%
    %%%%%%%%%%%%%%%%%%%%%%%%%%% POST-PROCESSING %%%%%%%%%%%%%%%%%%%%%%%%%%%
    %---------------------------------------------------------------------%   

    % Define equation for the pdf around a single primary quadrature point
    % The pdf is approximated using Eq. S34 of Dale et al. (2016)
    pdf = @(epsilon,t_index, i) w_vec(i,t_index)*betapdf((epsilon-min_m)/(max_m-min_m),x_trans_vec(i,t_index)./sigma_vec(1,t_index),(1-x_trans_vec(i,t_index))./sigma_vec(1,t_index))*(1/(max_m-min_m));

    % Calculate the raw fractional moments of the distribution from the pdf
    % at time t.  Note that the value at the first time step is an initial 
    % condition, so we do not need to solve for it here.
    for t_index = 2:n_t
     for k=1:num_moments
         for i=1:pts
            disp(t_index);
            mom_fun = @(epsilon) epsilon.^((k-1)/3).*( pdf(epsilon, t_index, i) ); 
            stats_frac(k,t_index) = stats_frac(k,t_index) + quadgk(mom_fun, min_m*1.0001, max_m*0.999);
         end
     end 
    end
    

    
    % Dissolution or 
    % Simultaneous aggregation and dissolution with fusion
    if(case_id == 1 || (case_id == 3 && case3_pass == 1))

        % Calculate the characteristic radius in m  according to Eq. S3 of 
        % Dale et al. (2016) (expressed in terms of particle mass instead 
        % of diameter by assuming spherical particles).  Note that the 
        % value at the first time step is an initial condition, so we do 
        % not need to solve for it here.
        for t_index=2:n_t
            for i=1:pts
                char_rad_fun = @(epsilon) epsilon.^(2/3)  .*  log((6/(rho_p*pi))^(1/3)*epsilon.^(1/3)) .*(pdf(epsilon, t_index, i)); 
                char_rad(1,t_index) = char_rad(1,t_index) + quadgk(char_rad_fun, min_m*1.0001, max_m*0.999);
            end
            char_rad(1,t_index) = exp(char_rad(1,t_index)./stats_frac(3,t_index))/2 ;
        end

        % Convert the characteristic radius in m to a diameter in nm
        stats_frac(2,:) = 2*(10^9)*char_rad(1,:); 
        
        % Use the geometry of a sphere to convert the raw 2/3 moment to the 
        % surface area concentration in m^2/m^3
        stats_frac(3,:) = stats_frac(3,:)*(4*pi)*((4/3)*rho_p*pi)^(-(2/3)); 
        
    % Aggregation
    elseif(case_id == 2)
        
        % Characteristic particle mass in g 
        % (from the geometry of a sphere)
        char_mass0 = (4/3)*pi*rho_p*(char_rad0^3); 
        
        % Characteristic particle surface area in m^2 
        % (from the geometry of a sphere)
        char_sa0 = 4*pi*(char_rad0^2); 

        % Calculate the geometric mean number of primary particles in the 
        % aggregate
        for t_index = 1:n_t
            for i=1:pts
                mean_num_per_fun = @(epsilon) log(epsilon./char_mass0).*(pdf(epsilon, t_index, i)); 
                mean_num_per(1,t_index) = mean_num_per(1,t_index)+integral(mean_num_per_fun, min_m*1.0001, max_m*0.999);
            end
        end
        mean_num_per(1,:)=exp(mean_num_per(1,:)./stats(1,:));

        % Characteristic diameter of the aggregate (in nm) as described by 
        % Eq. S10 of Dale et al. (2016)
        stats_frac(2,:) = char_rad0*((  mean_num_per(1,:)  ).^(1/Df))*2*(10^9); 
        
        % The surface area concentration (in m^2/m^3) does not change over 
        % time due to particle fusion in the absence of dissolution
        stats_frac(3,:) = N0*char_sa0; 
           
    % Simultaneous aggregation and dissolution, no fusion 
    elseif(case_id == 3 && case3_pass == 2)

        % Calculate the geometric mean number of primary particles in the 
        % aggregate
        for t_index = 1:n_t
            for i=1:pts
              mean_num_per_fun = @(epsilon) log(epsilon./char_mass_interp(1,t_index)).*(pdf(epsilon, t_index, i)); 
              mean_num_per(1,t_index) = mean_num_per(1,t_index)+integral(mean_num_per_fun, min_m*1.0001, max_m*0.999);
            end
        end
        mean_num_per(1,:)=exp(mean_num_per(1,:)./stats(1,:));
        
        % Characteristic diameter of the aggregate (in nm) as described by 
        % Eq. S10 of Dale et al. (2016)
        stats_frac(2,:) = char_rad_interp(1,:).*((  mean_num_per(1,:)  ).^(1/Df))*2*(10^9); 
        
        % The surface area concentration (in m^2/m^3) does not change over 
        % time due to particle fusion, only dissolution
        stats_frac(3,:) = N0.*char_sa_interp(1,:);  
    end

    % In all cases, the zeroth and 1st raw integer moments can be returned 
    % (re-indexed as the 0/3rds and 3/3rds raw fractional moments) without 
    % modification
    stats_frac(1,:) = stats(1,:);
    stats_frac(4,:) = stats(2,:);
    
    % Replace zero values with NaN so that they are not plotted
    stats_frac(stats_frac == 0) = NaN;

    return

end


% KEY REFERENCES
% For a complete list of references, see Dale et al. (2016)

% Yuan, C., F. Laurent, and R. Fox (2012), An extended quadrature method 
% of moments for population balance equations, Journal of Aerosol Science, 
% 51, 1-23.

% Young, T., and M. Mohlenkamp (2014), Introduction to Numerical Methods
% and MATLAB Programming for Engineers.  Code posted online as Lecture 6
% for Math 3600 (Applied Numerical Methods) at Ohio University:
% www.math.ohiou.edu/courses/math3600/lecture6.pdf

% Kelly, M.  Function ROOTSOLVE() posted to MathWorks FileExchange on 
% 11 Dec 2015.
% www.mathworks.com/matlabcentral/fileexchange/54458-rootsolve-func-xlow-xupp-/content/rootSolve.m

% Dette, H., and W. Studden (1997), The Theory of Canonical Moments with 
% Applications in Statistics, Probability, and Analysis. John Wiley & Sons.
% Hoboken, NJ.

