function [stats, char_rad, time] = method_DQMOM(case_id, sink, fusion, ...
    case3_pass, delt, delt2, t_switch, tvec, N0, S, rho_p, g_rat, ...
    k_S_pb, pow, prefac_agg, Df, ln_params, char_rad0, char_rad)

% METHOD_DQMOM tracks changes in statistics of the distribution over time 
% during dissolution, aggregation, or both using the Direct Quadrature 
% Method of Moments
%    
%   INPUTS:
%   case_id        - Test case flag:   
%                    1 -> Dissolution only
%                    2 -> Aggregation only 
%                    3 -> Simultaneous dissolution and aggregation
%   sink           - Sink flag:        
%                    0 -> Dissolution to equilibrium ion concentration
%                    1 -> Dissolution under sink conditions (no ion)
%   fusion         - Fusion flag:      
%                    0 -> Particles do not fuse upon aggregation
%                    1 -> Particles fuse completely upon aggregation
%   case3_pass     - Case 3 pass flag: 
%                    1 -> First pass (dissolution only)
%                    2 -> Second pass (dissolution & aggregation)
%
%   delt           - Time step prior to t_switch in days (case_id = 2 or 3)
%                    or for the entire simulation (case_id = 1)
%   delt2          - Time step after t_switch in days (case_id = 2 or 3)
%   t_switch       - Time in days at which the time step switches from delt 
%                    to delt2 (case_id = 2 or 3)
%   tvec           - Vector of the times in days at which to solve the
%                    statistics of interest for the current run
%
%   N0             - Initial particle number concentration in #/m^3
%   S              - Equilibrium ion concentration (solubility) in g/m^3
%   rho_p          - NP density in g/m^3
%   g_rat          - Stoichiometric coefficient for dissolution: g Me+
%                    generated per g MexOx dissolved
%   k_S_pb         - Dissolution rate
%   pow            - Power that the size coordinate is raised to in the
%                    rate law for dissolution
%   prefac_agg     - Prefactor for the aggregation rate
%   Df             - Assumed fractal dimension of aggregates
%   ln_params      - Vector containing the parameters of the initial
%                    lognormal nanoparticle size distribution: [mu, sigma]
%
%   char_rad       - Vector of the characteristic radius of primary 
%                    particles at every time step in m 
%                    (calculated and returned during the first pass of 
%                    case 3 and read in as input for the second pass)
%
%   OUTPUTS:
%   stats          - Matrix of statistics of the distribution as a function
%                    of time (calculated from the raw fractional moments) 
%                    to be returned to CONTROL_FILE.m for visualization and 
%                    error analysis
%   char_rad       - Defined in INPUTS
%   time           - Simulation runtime in s


    %---------------------------------------------------------------------%
    %%%%%%%%%%%%%%% MODEL RESOLUTION (USER-SPECIFIED INPUT) %%%%%%%%%%%%%%%
    %---------------------------------------------------------------------%
    % Number of points for n-point quadrature in DQMOM  
    % Note: Beyond N = 5 to 6, numerical problems are likely
    % N must be at least 2 to predict surface area and mass concentrations
    if(case_id == 1)
        pts = 2;
    elseif(case_id == 2)
        pts = 2;
    elseif(case_id == 3)
        pts = 2;
    end

    % rat_cnstr = 0 --> DQMOM without ratio constraints
    % rat_cnstr = 1 --> DQMOM with ratio constraints
    % where ratio constraints are defined as per Fox et al. (2008)
    rat_cnstr = 1;
    
    % Use of ratio contraints is our default
    if(case_id == 2 || case_id == 3)  
        rat_cnstr = 1;  
    end
    
    %---------------------------------------------------------------------%
    %%%%%%%%%%%%%%%%%%%%%%%%%%% INITIALIZATION %%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %---------------------------------------------------------------------%
    
    % Number of time steps in the simulation
    n_t = length(tvec);
    
    % Number of lower-order moments that can be solved for exactly with pts
    % quadrature points
    k_max = 2*pts - 1; 

    % Initially holds the raw fractional moments but is later converted
    % to the desired statistics of the distribution (calculated from the 
    % raw moments) that are returned by the function
    stats = zeros(k_max+1,n_t);
    
    % Holds the ion mass concentration in g/m^3 at each time step
    mass_conc_ion = zeros(1,n_t);
    
    % Holds the weights, absicassas, and weighted abscissas of the 
    % distribution for a given time step
    w = zeros(pts);
    x=zeros(pts);
    x_wtd=zeros(pts);
    
    % Holds the change in the weights and the weighted absicassas
    % during a timestep due to dissolution or aggregation
    a_diss = 0;
    b_diss = 0;
    a_agg = 0;
    b_agg = 0;
    
    % char_rad --> Holds the characteristic (surface-weighted geometric
    % mean) radius at every time step
    % If case3_pass == 2, char_rad has been passed to the function from 
    % the first pass and should not be reset.
    if(case_id == 1 || (case_id == 3 && case3_pass == 1))
        
        char_rad = zeros(1,n_t); 
        
    elseif(case_id == 2)
        % From the geometry of a sphere ...
        
        % Characteristic particle mass in g 
        char_mass0 = (4/3)*pi*rho_p*(char_rad0^3);  

        % Characteristic particle surface area in m^2 
        char_sa0 = 4*pi*(char_rad0^2); 
        
    elseif(case_id == 3 && case3_pass == 2)
        % From the geometry of a sphere ...
        
        % Holds the characteristic particle mass 
        char_mass(1,:) = rho_p*(4/3)*pi*(char_rad(1,:)).^3;  
        
        % Holds the characteristic particle surface area in m^2
        char_sa(1,:) = 4*pi*(  char_rad(1,:)  ).^2;            
    end
    
    
    % Holds the geometric mean number of primary particles in an aggregate 
    % (if applicable)
    mean_num_per = zeros(1,n_t);  

    % When the initial distribution is lognormal, the initial moments of
    % the distribution can be calculated exactly from the parameters of the
    % distribution using Eq. S48 of Dale et al. (2016)
    for k=1:k_max+1
        stats(k,1) =  N0*exp(  ((k-1)/3)*ln_params(1)  + ((((k-1)/3)^2*(ln_params(2)^2))/2)   );
    end
    
    %---------------------------------------------------------------------%
    %%%%%%%%%%%%%%%%%%%%%%%%%% NUMERICAL SOLUTION %%%%%%%%%%%%%%%%%%%%%%%%%
    %---------------------------------------------------------------------%
    
    tic % Start timer

    % For every time step ...
    for t_index = 1:n_t-1

        % Grab the time in days for the current time step
        t = tvec(t_index);

        % Grab the appropriate time step (before or after t_switch)
        if((case_id == 2 || case_id == 3) && t >= t_switch)
            delt = delt2;
        end

        % Print time at which we are solving in days
        disp(t);

         % At the first time step, use a matrix inversion algorithm like
         % Product Difference to calculate the initial weights and 
         % abscissas of the distribution
         if(t_index == 1)

            % This returns the initial weights and (unweighted) abscissas
            [w,x,werror] = ALL_productDifference(0,pts,stats(:,t_index));
            w = w';  x = x';
            
            if(werror)
                error('There was an error in DQMOM during the matrix inversion step.')
            end
            
            % Use of fractional moments (k = 0, 1/3, 2/3, 1 ...) rather 
            % than integer moments requires this simple coordinate 
            % transformation
            x=x.^3; 
            
            x_wtd = x.*w; % Weight the absiccas
            
            % Calculate useful statistics of the initial distribution
             for k = 1:2*pts
                if(case_id == 1 || (case_id == 3 && case3_pass == 1))
                   % Calculate the characteristic radius in m
                   char_rad(1,t_index) = exp(sum(w.*(x.^(2/3)).*log((((6/(rho_p*pi))^(1/3)))*x.^(1/3)))/sum(w.*(x.^(2/3))))/2; 
                elseif(case_id == 2)
                    % Geometric mean number of primary particles in the aggregate
                    mean_num_per(1,t_index) = exp( sum(w.*log(x / char_mass0))./stats(1,t_index)   );  
                elseif(case_id == 3 && case3_pass == 2)
                    % Geometric mean number of primary particles in the aggregate
                    mean_num_per(1,t_index) = exp( sum(w.*log(x / char_mass(1,t_index)))./stats(1,t_index)   ); 
                end
             end
         end

         % If dissolution occurs and ratio contraints are specified ...
         % Use the rate law (A_i, Eq. S6 in Dale et al. (2016)) to 
         % calculate the change in the weights and the weighted abscissas 
         % from one time step to the next using the ratio constraint 
         % approach by Fox et al. (2008)
         if( case_id ~= 2 && rat_cnstr == 1 ) 
             if(case_id == 1 || (case_id == 3 && case3_pass == 1))
                 A_i = delt*k_S_pb*(mass_conc_ion(1,t_index)-S)*x.^pow;
                 [~,a_diss,b_diss] = DQMOM_dissFlux(pts,x,w,A_i);

             elseif(case_id == 3 && case3_pass == 2)
                 A_i = delt*k_S_pb*(x./char_mass(1,t_index))*(mass_conc_ion(1,t_index)-S)*char_mass(1,t_index).^pow;
                 [~,a_diss,b_diss] = DQMOM_dissFlux(pts,x,w,A_i);
             end
         end
         
         % If ratio constraints are not specified or aggregation occurs,
         % Evolve the weights and abscissas using the matrix approach
         % described in the Appendix of Marchisio and Fox (2005),
         % which solves the linear system of equations given as Eq. S27 in
         % Dale et al. (2016)
         if(rat_cnstr == 0 || case_id == 2 || (case_id == 3 && case3_pass == 2) || (case_id == 3 && fusion == 1))
             
            %A1, A2, and d are defined as per Marchisio and Fox (2005)
            A1 = zeros(2*pts,pts);
            A2 = zeros(2*pts,pts);
            d_diss = zeros(2*pts); 
            d_agg = zeros(2*pts);
    
             %define a scaling factor to improve numerical stability
             x_scale = max(x);

             % Set up A (with rescaling)
             for i = 1:2*pts
                 for j = 1:pts
                     A1(i,j)=((1-((i-1)/3))*(x(j))^((i-1)/3))./(x_scale.^((i-1)/3));
                     A2(i,j)=(((i-1)/3)*(x(j))^(((i-1)/3)-1))./(x_scale.^((i-1)/3));
                 end
             end      

             A = [A1 A2];

             
             if(rat_cnstr == 0) % Dissolution without ratio constraints
                 
                 % Set up d (with rescaling)
                 for k = 1:2*pts
                    d_diss(k) = 0;
                    for a = 1:pts
                        % d_diss describes the evolution of the weights and
                        % abscissas due to dissolution and thus includes
                        % the source term Sk,diss as defined in Eq. S27 of
                        % Dale et al. (2016)
                        d_diss(k) = d_diss(k) + (((k-1)/3)*k_S_pb*(mass_conc_ion(1,t_index)-S).*  (x(a).^pow).*(x(a)^(((k-1)/3)-1))*w(a))./(x_scale.^((k-1)/3));
                    end
                 end
                 
                 % Invert the matrix to find the change in the weights and
                 % the weighted abscissas due to dissolution
                 alpha = A\d_diss(:,1);
                 a_diss = delt*alpha(1:pts);
                 b_diss = delt*alpha(pts+1:2*pts);
                 
             else % Aggregation
                % set up d with rescaling
                 for k = 1:2*pts
                    d_agg(k) = 0;
                    for a = 1:pts
                        for g = 1:pts
                            % d_agg describes the evolution of the weights
                            % and abscissas due to aggregation and thus
                            % includes the source term Sk,agg as defined in 
                            % Eq. S27 of Dale et al. (2016)
                            beta_ag = prefac_agg*( ((x(a)^(1/Df) + x(g)^(1/Df))^2) / ((x(a)*x(g))^(1/Df)) );
                            d_agg(k) = d_agg(k) - (0.5*beta_ag*w(a)*w(g)*(x(a)^((k-1)/3)+x(g)^((k-1)/3)-((x(a)+x(g)).^((k-1)/3))))./(x_scale.^((k-1)/3));
                        end
                    end
                 end
                 
                 % Invert the matrix to find the change in the weights and
                 % the weighted abscissas due to aggregation
                 alpha = A\d_agg(:,1);
                 a_agg = delt*alpha(1:pts);
                 b_agg = delt*alpha(pts+1:2*pts);
             end

         end

         % Evolve the weights and weighted abscissas from time t to time
         % t+delt
         w = w + a_diss + a_agg;
         x_wtd = x_wtd + b_diss + b_agg;   

         % Un-weight the absiccas
         x = x_wtd./w;


         stats(k,t_index+1)=0;
         for k = 1:2*pts

            % Calculate the raw fractional moments of the distribution
            % from their quadrature points (Eq. S12 of Dale et al. (2016))
            stats(k,t_index+1) = sum(w.*(x.^((k-1)/3)));

            % Calculate the geometric mean number of primary particles in 
            % the aggregate and their characteristic radius
            if(case_id == 1 || (case_id == 3 && case3_pass == 1))
                char_rad(1,t_index+1) = exp(sum(w.*(x.^(2/3)).*log((((6/(rho_p*pi))^(1/3)))*x.^(1/3)))/sum(w.*(x.^(2/3))))/2;
            elseif(case_id == 2)
                mean_num_per(1,t_index+1) = exp( sum(w.*log(x / char_mass0)) ./stats(1,t_index+1)  );
            elseif(case_id == 3 && case3_pass == 2)
                mean_num_per(1,t_index+1) = exp( sum(w.*log(x / char_mass(1,t_index+1)))./stats(1,t_index+1) ); 
            end
         end

         % If the absiccas drop below zero, set the corresponding weights
         % to zero (particles have fully dissolved)
         w(x < 0) = 0;

         % Calculate changes in the ion concentration during the time step
         % due to dissolution
         if((case_id == 1 || case_id == 3) && sink == 0)
           diff = stats(4,t_index)-stats(4,t_index+1);
           mass_conc_ion(1,t_index+1) = mass_conc_ion(1,t_index)+g_rat*diff;  
         end
         
    end    %end time loop 

    time = toc; % End timer

    
    %---------------------------------------------------------------------%
    %%%%%%%%%%%%%%%%%%%%%%%%%%% POST-PROCESSING %%%%%%%%%%%%%%%%%%%%%%%%%%%     
    %---------------------------------------------------------------------%
    
    % Dissolution, or 
    % Simulataneous aggregation and dissolution with fusion
    if(case_id == 1 || (case_id == 3 && fusion == 1)) 
        
        % Convert the characteristic radius in m to a diameter in nm
        stats(2,:) = 2*(10^9)*char_rad(1,:);
        
        % Use the geometry of a sphere to convert the raw 2/3 moment to the 
        % surface area concentration in m^2/m^3
        stats(3,:) = stats(3,:)*(4*pi)*((4/3)*rho_p*pi)^(-(2/3));  
        
    % Aggregation
    elseif(case_id == 2) 
        % Characteristic radius of the aggregate (in nm) as described by 
        % Eq. S10 of Dale et al. (2016)
        stats(2,:)=char_rad0*((  mean_num_per(1,:)  ).^(1/Df))*2*(10^9); 
        
        % The surface area concentration (in m^2/m^3) does not change over 
        % time due to particle fusion in the absence of aggregation
        stats(3,:)=N0*char_sa0; 
                
    % Simultaneous aggregation and dissolution, no fusion
    elseif(case_id == 3 && case3_pass == 2) 
        
        % Characteristic radius of the aggregate (in nm) as described by 
        % Eq. S10 of Dale et al. (2016)
        stats(2,:)=char_rad(1,:).*((  mean_num_per(1,:)  ).^(1/Df))*2*(10^9); 
        
        % The surface area concentration (in m^2/m^3) does not change over 
        % time due to particle fusion, only dissolution
        stats(3,:)=N0.*char_sa(1,:);  
        
    end
    
    % Replace zero values with NaN so that they are not plotted
    stats(stats == 0) = NaN;
    
    return

end


% KEY REFERENCES
% For a complete list of references, see Dale et al. (2016)

% Fox, R. O., F. Laurent, and M. Massot (2008), Numerical simulation of 
% spray coalescence in an Eulerian framework: direct quadrature method of 
% moments and multi-fluid method, Journal of Computational Physics, 
% 227(6), 3058-3088.

% Marchisio, D. L., and R. O. Fox (2005), Solution of population balance 
% equations using the direct quadrature method of moments, Journal of 
% Aerosol Science, 36(1), 43-73.
