function [stats, tvec, char_rad, time] = method_DSMC(case_id, sink, ...
    fusion, case3_pass, num_moments, n_days, delt, tvec_pre, N0, S, ...
    rho_p, g_rat, k_S_pb, pow, prefac_agg, Df, ln_params, char_rad0, ...
    char_rad)
    
% METHOD_DSMC tracks changes in statistics of the distribution over time 
% during dissolution, aggregation, or both using Direct Simulation Monte
% Carlo
%    
%   INPUTS:
%   case_id        - Test case flag:   
%                    1 -> Dissolution only
%                    2 -> Aggregation only 
%                    3 -> Simultaneous dissolution and aggregation
%   sink           - Sink flag:        
%                    0 -> Dissolution to equilibrium ion concentration
%                    1 -> Dissolution under sink conditions (no ion)
%   fusion         - Fusion flag:      
%                    0 -> Particles do not fuse upon aggregation
%                    1 -> Particles fuse completely upon aggregation
%   case3_pass     - Case 3 pass flag: 
%                    1 -> First pass (dissolution only)
%                    2 -> Second pass (dissolution & aggregation)
%
%   num_moments    - Number of raw fractional moments for which to solve
%   n_days         - Number of days in the simulation
%   delt           - Time step in days
%   tvec_pre       - Vector of the simulation time at every time step in
%                    days for the first case 3 pass, if on the second
%
%   N0             - Initial particle number concentration in #/m^3
%   S              - Equilibrium ion concentration (solubility) in g/m^3
%   rho_p          - NP density in g/m^3
%   g_rat          - Stoichiometric coefficient for dissolution: g Me+
%                    generated per g MexOx dissolved
%   k_S_pb         - Dissolution rate
%   pow            - Power that the size coordinate is raised to in the
%                    rate law for dissolution
%   prefac_agg     - Prefactor for the aggregation rate
%   Df             - Assumed fractal dimension of aggregates
%   ln_params      - Vector containing the parameters of the initial
%                    lognormal nanoparticle size distribution: [mu, sigma]
%
%   char_rad       - Vector of the characteristic radius of primary 
%                    particles at every time step in m 
%                    (calculated and returned during the first pass of 
%                    case 3 and read in as input for the second pass)
%
%   OUTPUTS:
%   stats          - Matrix of statistics of the distribution as a function
%                    of time (calculated from the raw fractional moments) 
%                    to be returned to CONTROL_FILE.m for visualization and 
%                    error analysis
%   tvec           - Vector of the simulation time at every time step in
%                    days for the first case 3 pass, returned and read in
%                    as input on the second pass
%   char_rad       - Defined in INPUTS
%   time           - Simulation runtime in s

    %---------------------------------------------------------------------%
    %%%%%%%%%%%%%%% MODEL RESOLUTION (USER-SPECIFIED INPUT) %%%%%%%%%%%%%%%
    %---------------------------------------------------------------------%

    % sample_n -> Size of the representative sample (# of particles)
    % f -> safety factor to use in calculating the time step for
    % aggregation
    if(case_id == 1)
        sample_n = 3e4;
    elseif(case_id == 2)
        sample_n = 3e3;     
        f=0.05;
    elseif(case_id == 3 && case3_pass == 1)
        sample_n = 1e3;
        f=0.05;
    elseif(case_id == 3 && case3_pass == 2)
        sample_n = 1e3;
        f=0.05;
    end
    
    

    %---------------------------------------------------------------------%
    %%%%%%%%%%%%%%%%%%%%%%%%%%% INITIALIZATION %%%%%%%%%%%%%%%%%%%%%%%%%%%% 
    %---------------------------------------------------------------------%    
    
    % Specify the initial mass of sample_n particles by drawing randomly 
    % from the initial distribution
    sample=lognrnd(ln_params(1),ln_params(2),1,sample_n); 

    % The sample is only a representative fraction frac of the entire 
    % population
    frac = N0/sample_n;
    
    % Specify an initial simulation volume for the sample (m3)
    % Setting V0 = 1/frac ensures that the volume taken up by the entire
    % population (not just the representative fraction) is 1 m^3
    V0 = 1/frac;
    
    % The solution volume will change over time but is initially V0
    V_sol = V0; 

 
    % Calculate the initial statistics of the distribution:
    % stats -> Holds the desired statistics of the distribution (calculated 
    % from the raw moments) that are returned by the function
    % char_rad -> Characteristic (surface-weighted geometric mean) radius
    % in m
    % char_mass -> Characteristic particle mass in g
    % char_sa -> Characteristic particle surface area in m^2
    % mean_num_per -> Geometric mean number of primary particles in an 
    % aggregate 
    for k = 1:num_moments
        % First calculate the raw fractional moments
        stats(k,1) = sum(sample.^((k-1)/3))*frac;   
    end

    if(case_id == 1 || (case_id == 3 && case3_pass == 1))
        % Eq. S3 of Dale et al. (2016) (expressed in terms of particle mass
        % instead of diameter by assuming spherical particles)
        char_rad = exp( sum( (sample.^(2/3)).*log((((6/(rho_p*pi))^(1/3))).*sample.^(1/3)) ) / sum(sample.^(2/3)))/2;
        
        stats(2,1) = 2*(10^9)*char_rad; %radius in m to diameter in nm
        
        % Use the geometry of a sphere to convert the raw 2/3 moment to the 
        % surface area concentration in m^2/m^3
        stats(3,1) = stats(3,1)*(4*pi)*((4/3)*rho_p*pi)^(-(2/3));
    elseif(case_id == 2)
        % From the geometry of a sphere
        char_mass0 = (4/3)*pi*rho_p*(char_rad0^3); 
        char_sa0 = 4*pi*(char_rad0^2);
        
        %Eq. S10 of Dale et al. (2016)
        stats(2,1) = char_rad0*(( exp((sum(log(sample/char_mass0))*frac)/N0) ).^(1/Df))*2*(10^9);  
        
        % The surface area concentration (in m^2/m^3) does not change over 
        % time due to particle fusion in the absence of dissolution
        stats(3,1) = N0*char_sa0;
    elseif(case_id == 3 && case3_pass == 2)  
        % From the geometry of a sphere
        char_mass = rho_p*(4/3)*pi*(char_rad(1,1)).^3; 
        char_sa =   4*pi*(char_rad(1,1))^2;
        
        mean_num_per = exp(sum(log(sample ./ char_mass(1,1)))./N0); 
        
        %Eq. S10 of Dale et al. (2016)
        stats(2,1) = char_rad(1,1).*(mean_num_per^(1/Df))*2e9;  
        
        % The surface area concentration (in m^2/m^3) does not change over 
        % time due to particle fusion, only dissolution
        stats(3,1) = N0.*char_sa(1,1);
    end

    % Will hold the ion mass concentration in g/m^3
    if(sink == 0)
        mass_conc_ion = 0;
    end
    
    % If aggregation occurs ...
    % Holds the mean time to coagulation for each particle (t_coagi)
    % and the sum over all values of beta that is used to calculate
    % t_coagi (denominator of Eq. S22 of Dale et al. (2016))
    if(case_id == 2 || (case_id == 3 && case3_pass == 2) || (case_id == 3 && fusion == 1))
        t_coagi=zeros(1,sample_n);
        sum_betai=zeros(1,sample_n);
    end

    
    %---------------------------------------------------------------------%
    %%%%%%%%%%%%%%%%%%%%%%%%%% NUMERICAL SOLUTION %%%%%%%%%%%%%%%%%%%%%%%%%
    %---------------------------------------------------------------------%
    % The numerical solution is the Direct Simulation Monte Carlo approach
    % presented by Zhao et al. (2007) as described by Dale et al. (2016)
    
    tic % Start timer

    % We will step through the simulation rather than pre-specifying tvec 
    % so that we can use a variable time step
    tvec = 0; 
    t_index = 1;
    while tvec(t_index) < n_days
        
        % Print time at which we are solving in days
        disp(tvec(t_index));
        
        % If aggregation occurs, calculate the next time step using
        % Eq. S22 of Dale et al. (2016)
        if(case_id == 2 || (case_id == 3 && case3_pass == 2) || (case_id == 3 && fusion == 1))
            for i = 1:length(sample)

                sum_betai(i)=0;
                for j = 1:length(sample)
                    if(i ~= j)
                        sum_betai(i) = sum_betai(i) + prefac_agg*( ((sample(i)^(1/Df) + sample(j)^(1/Df))^2) / ((sample(i)*sample(j))^(1/Df)) );
                    end
                end

                t_coagi(i)=V_sol/sum_betai(i);
            end

            tvec(t_index+1) = tvec(t_index) + f*min(t_coagi);
        else
            tvec(t_index+1) = tvec(t_index)+delt;
        end

        % In the case of simultaneous aggregation and dissolution, 
        % the initial "dissolution only" pass can have a different time
        % step than the final 'dissolution and aggregation' pass (to
        % improve model performance).  In this case, interpolate values of 
        % the characteristic radius between time steps.
        if(case_id == 3 && case3_pass == 2)
            
            % Find the index for the closest time step in the first pass
            sub_vec = tvec_pre-tvec(t_index+1);
            [min_val, min_ind]=min(abs(sub_vec));

            % If the times are the same for both passes or we are at the 
            % boundary of tvec_pre ...
            if(sub_vec(min_ind)==0 || (sub_vec(min_ind) > 0 && min_ind == 1) || (sub_vec(min_ind) < 0 && min_ind >= length(tvec_pre)-1)  )
                char_rad_interp = char_rad(min_ind);
                
            % If the closest time step from the first pass is larger than
            % the time step for the second pass, interpolate between that
            % time step and its next smallest neighbor     
            elseif(sub_vec(min_ind) > 0)
                char_rad_interp = char_rad(min_ind-1) + ((tvec(t_index+1)-tvec_pre(min_ind-1))*(char_rad(min_ind)-char_rad(min_ind-1)))/(tvec_pre(min_ind)-tvec_pre(min_ind-1));
                
            % If the closest time from the first pass is smaller than
            % the time step for the second pass, interpolate between that 
            % time step and its next largest neighbor    
            elseif(sub_vec(min_ind) < 0)
                char_rad_interp = char_rad(min_ind) + ((tvec(t_index+1)-tvec_pre(min_ind))*(char_rad(min_ind+1)-char_rad(min_ind)))/(tvec_pre(min_ind+1)-tvec_pre(min_ind));
            end
            
            % Calculate the characteristic particle mass and surface area
            % (from the geometry of a sphere)
            char_mass_interp = rho_p*(4/3)*pi*(char_rad_interp).^3;
            char_sa_interp =   4*pi*(char_rad_interp)^2;

        end

        % Dissolve the particles according to their rate law
        if(case_id == 1 || case_id == 3)
            part_i = 1;
            while part_i < length(sample)
                if(sink == 0)
                    ion_grad = S-mass_conc_ion(1,end);
                else
                    ion_grad = S;
                end
                
                if(case_id == 3 && case3_pass == 2)
                    sample(part_i)=sample(part_i) - (sample(part_i)/char_mass_interp)*(f*min(t_coagi))*k_S_pb*ion_grad*(char_mass_interp^pow);
                elseif(case_id == 3 && fusion == 1)
                    sample(part_i)=sample(part_i)-(f*min(t_coagi))*k_S_pb*ion_grad*(sample(part_i)^pow);   
                else
                    sample(part_i)=sample(part_i)-delt*k_S_pb*ion_grad*(sample(part_i)^pow); 
                end
                
                % If a particle completely dissolves, remove it from sample
                % and also remove its index from t_coagi and sum_betai
                if(sample(part_i) < 0)
                    sample(part_i) = [];
                    if((case_id == 3 && case3_pass == 2) || (case_id == 3 && fusion == 1))
                        t_coagi(part_i)=[];
                        sum_betai(part_i)=[];  
                    end
                end
                
                part_i = part_i + 1;  
            end
        end

        % If aggregation occurs ...
        if(case_id == 2 || (case_id == 3 && case3_pass == 2) || (case_id == 3 && fusion == 1))
            part_i = 1;
            while part_i < length(sample)
                % Use Eq. S23 of Dale et al. (2016) to decide whether it
                % aggregates during the time step.  If so ...
                if( rand() < 1 - exp(-  (f*min(t_coagi)) / (2*t_coagi(part_i))) )

                    breakcond = rand(); % break condition
                    Pij = 0;

                    % Use Eq. S24 of Dale et al. (2016) to find the particle
                    % j with which particle i will aggregate
                    part_j = 0;
                    while( (part_j <= length(sample)-1) && (Pij < breakcond) )
                        part_j = part_j+1;

                        if(part_i ~= part_j)
                            Pij = Pij + (prefac_agg*( ((sample(part_i)^(1/Df) + sample(part_j)^(1/Df))^2) / ((sample(part_i)*sample(part_j))^(1/Df)) ))/sum_betai(part_i);
                        end
                    end

                    % Aggregate the particles by summing their masses and
                    % removing particle j from the sample
                    sample(part_i)=sample(part_i)+sample(part_j);
                    sample(part_j)=[];
                    t_coagi(part_j)=[];
                    sum_betai(part_j)=[];            
                    
                end  
                part_i = part_i + 1; 
            end
        end

        % If the number of particles in the sample drops below half of its
        % initial value ...
        if(length(sample) < sample_n/2)

            % ... double the sample
            sample=[sample sample];

            % ... then double the solution volume to keep the number 
            % concentration fixed      
            V_sol = 2*V_sol;
        end 

        % Calculate the statistics of the distribution at the new time step
        % See comments above (for the initial time step) for details
        for k = 1:num_moments
            stats(k,t_index+1) = sum(sample.^((k-1)/3))*frac*(V0/V_sol);
        end

        if(case_id == 1 || (case_id == 3  && case3_pass == 1))
            char_rad(1,end+1) = exp(sum(   (sample.^(2/3)).*log((((6/(rho_p*pi))^(1/3))).*sample.^(1/3))     )/     sum(sample.^(2/3))        )/2;
            stats(2,t_index+1) = 2*(10^9)*char_rad(1,end);
            stats(3,t_index+1) = stats(3,t_index+1)*(4*pi)*((4/3)*rho_p*pi)^(-(2/3));
        elseif(case_id == 2)
            stats(2,t_index+1) = char_rad0*(( exp((sum(log(sample./char_mass0)).*frac*(V0/V_sol))./stats(1,t_index+1))  ).^(1/Df))*2*(10^9); 
            stats(3,t_index+1) = N0*char_sa0;
        elseif(case_id == 3 && case3_pass == 2)
            mean_num_per = exp(sum(log(sample ./ char_mass_interp))/(stats(1,t_index+1)/(frac*(V0/V_sol)))); 
            stats(2,t_index+1)=char_rad_interp.*(mean_num_per^(1/Df))*2e9;
            stats(3,t_index+1) = N0.*char_sa_interp;
        end

         if((case_id == 1 || case_id == 3) && sink == 0)
               diff = stats(4,t_index+1-1)-stats(4,t_index+1);
               mass_conc_ion(1,t_index+1) = mass_conc_ion(1,t_index)+g_rat*diff;  
         end

         if(  (case_id == 3 && case3_pass == 2) && stats(4,t_index+1) < 0.05)
            break;
         end
         
         t_index = t_index + 1;
    end  
    
    time = toc; % End timer
    
    return

end

% KEY REERENCES
% For a complete list of references, see Dale et al. (2016)

% Zhao, H., A. Maisels, T. Matsoukas, and C. Zheng (2007), Analysis of four
% Monte Carlo methods for the solution of population balances in dispersed
% systems, Powder Technology, 173(1), 38-50.
