function [stats, char_rad, time] = method_SM(case_id, sink, fusion, ...
    case3_pass, num_moments, delt, delt2, t_switch, tvec, N0, S, ...
    rho_p, g_rat, k_S_pb, pow, prefac_agg, Df, ln_params, char_rad0, ...
    min_m, max_m, char_rad, mu_d0)

% METHOD_SM tracks changes in statistics of the distribution over time 
% during dissolution, aggregation, or both using the Sectional Method
%    
%   INPUTS:
%   case_id        - Test case flag:   
%                    1 -> Dissolution only
%                    2 -> Aggregation only 
%                    3 -> Simultaneous dissolution and aggregation
%   sink           - Sink flag:        
%                    0 -> Dissolution to equilibrium ion concentration
%                    1 -> Dissolution under sink conditions (no ion)
%   fusion         - Fusion flag:      
%                    0 -> Particles do not fuse upon aggregation
%                    1 -> Particles fuse completely upon aggregation
%   case3_pass     - Case 3 pass flag: 
%                    1 -> First pass (dissolution only)
%                    2 -> Second pass (dissolution & aggregation)
%
%   num_moments    - Number of raw fractional moments for which to solve
%   delt           - Time step prior to t_switch in days (case_id = 2 or 3)
%                    or for the entire simulation (case_id = 1)
%   delt2          - Time step after t_switch in days (case_id = 2 or 3)
%   t_switch       - Time in days at which the time step switches from delt 
%                    to delt2 (case_id = 2 or 3)
%   tvec           - Vector of the times in days at which to solve the
%                    statistics of interest for the current run
%
%   N0             - Initial particle number concentration in #/m^3
%   S              - Equilibrium ion concentration (solubility) in g/m^3
%   rho_p          - NP density in g/m^3
%   g_rat          - Stoichiometric coefficient for dissolution: g Me+
%                    generated per g MexOx dissolved
%   k_S_pb         - Dissolution rate
%   pow            - Power that the size coordinate is raised to in the
%                    rate law for dissolution
%   prefac_agg     - Prefactor for the aggregation rate
%   Df             - Assumed fractal dimension of aggregates
%   ln_params      - Vector containing the parameters of the initial
%                    lognormal nanoparticle size distribution: [mu, sigma]
%   min_m          - Lower bound of the size domain in m
%   max_m          - Upper bound of the size domain in m
%
%   char_rad       - Vector of the characteristic radius of primary 
%                    particles at every time step in m 
%                    (calculated and returned during the first pass of 
%                    case 3 and read in as input for the second pass)
%
%   OUTPUTS:
%   stats          - Matrix of statistics of the distribution as a function
%                    of time (calculated from the raw fractional moments) 
%                    to be returned to CONTROL_FILE.m for visualization and 
%                    error analysis
%   char_rad       - Defined in INPUTS
%   time           - Simulation runtime in s

    
    %---------------------------------------------------------------------%
    %%%%%%%%%%%%%%% MODEL RESOLUTION (USER-SPECIFIED INPUT) %%%%%%%%%%%%%%%
    %---------------------------------------------------------------------%                                                                                                                                            -

    % q determines the resolution of the size grid (placement of size bins)
    % as described by Lister et al. (1995) and given in Dale et al. (2016) 
    % [Eq. S16].  For aggregation, q must be an integer.
    if(case_id == 1)
        if(mu_d0 >= 50)
            q = 5;   
        else
            q = 7;
        end
    elseif(case_id == 2)
        q = 3;
    elseif(case_id == 3)
        q = 6;
    end
    
    %---------------------------------------------------------------------%
    %%%%%%%%%%%%%%%%%%%%%%%%%%% INITIALIZATION %%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %---------------------------------------------------------------------%
    
    % Number of time steps in the simulation
    n_t = length(tvec);
    
    m_rat = 2^(1/q); % Ratio of masses between adjoining bins (Eq. S16)

    % Number of size bins within the defined domain (plus one for indexing 
    % from 1 intead of zero, and plus two for empty bins at the top and 
    % bottom boundaries). 
    % Eq. S18 of Dale et al. (2016)
    n_bins = ceil(1+( log(max_m/min_m) / log(m_rat) ))+3;

    % Initially holds the raw fractional moments but is later converted
    % to the desired statistics of the distribution (calculated from the 
    % raw moments) that are returned by the function
    stats = zeros(num_moments,n_t);
    
    % Holds the mass at the lower boundary of each bin in g
    bin_i_lo_m = zeros(n_bins,1);
    
    % Holds the mass to be used as the midpoint of each bin, 
    % which is a function of the moment
    bin_i_mid_mom = zeros(num_moments,n_bins); 
    
    % Holds the number concentration of particles in each bin at each 
    % time step in #/m^3
    num_conc = zeros(n_bins, n_t);  
    
    % Holds the ion mass concentration in g/m^3 at each time step
    if(sink == 0)
        mass_conc_ion = zeros(1,n_t);  
    end
    
    % char_rad --> Holds the characteristic (surface-weighted geometric
    % mean) radius at every time step
    % If case3_pass == 2, char_rad has been passed to the function from 
    % the first pass and should not be reset.
    if(case_id == 1 || (case_id == 3 && case3_pass == 1))
        
        char_rad = zeros(1,n_t); 
      
    elseif(case_id == 2)
        % From the geometry of a sphere ...
        
        % Characteristic particle mass in g 
        char_mass0 = (4/3)*pi*rho_p*(char_rad0^3);  

        % Characteristic particle surface area in m^2 
        char_sa0 = 4*pi*(char_rad0^2); 
        
    elseif(case_id == 3 && case3_pass == 2)
        % From the geometry of a sphere ...
        % Holds the characteristic particle mass 
        char_mass(1,:) = rho_p*(4/3)*pi*(char_rad(1,:)).^3;  
        
        % Holds the characteristic particle surface area in m^2
        char_sa(1,:) = 4*pi*(  char_rad(1,:)  ).^2;            
    end
    
    % Holds the geometric mean number of primary particles in an aggregate 
    % (if applicable)
    mean_num_per = zeros(1,n_t); 

    % Calculate the mass at the lower boundary of each bin (in g) 
    % as given by Kostoglou and Karabelas (1995)
    % Eq. S17 of Dale et al. (2016)
    bin_i_lo_m(1,1) = min_m/2;
    bin_i_lo_m(2,1)= min_m;
    for bin_i = 3:n_bins
        bin_i_lo_m(bin_i,1) = (m_rat^(bin_i-2))*min_m; 
    end
    
    % Calculate the mass to use as the midpoint of the bin as given by
    % Hounslow et al. (1988) and Kostoglou and Karabelas (1995) 
    % Eq. S19 of Dale et al. (2016)
    for k = 1:num_moments
        for bin_i = 1:n_bins
          bin_i_mid_mom(k, bin_i) = ((m_rat^(((k-1)/3)+1)-1)/((((k-1)/3)+1)*(m_rat-1)))*(bin_i_lo_m(bin_i,1).^((k-1)/3));
        end
    end

    % The number concentration in each bin is the cdf for the upper end of 
    % the bin minus the cdf for the lower end of the range
    % The formula for the cdf of a lognormal distribution is Eq. S47 of Dale
    % et al. (2016)
    for bin_i = 2:(n_bins-2)
        
        % mass at the lower boundary of the bin
        lo = bin_i_lo_m(bin_i,1); 
        
        % mass at the upper boundary of the bin
        hi = m_rat*bin_i_lo_m(bin_i,1); 

        num_conc(bin_i, 1) = N0*( (1/2)*erfc(  - (log(hi)-ln_params(1)) / (ln_params(2)*sqrt(2))   ) ...
                                - (1/2)*erfc(  - (log(lo)-ln_params(1)) / (ln_params(2)*sqrt(2))   ) );
    end


    %---------------------------------------------------------------------%
    %%%%%%%%%%%%%%%%%%%%%%%%%% NUMERICAL SOLUTION %%%%%%%%%%%%%%%%%%%%%%%%%
    %---------------------------------------------------------------------%
    
    tic % Start timer

    % For every time step ...
    for t_index = 1:n_t    

        % Grab the time in days for the current time step
        t = tvec(t_index);

        % Grab the appropriate time step (before or after t_switch)
        if((case_id == 2 || case_id == 3) && t >= t_switch)
            delt = delt2;
        end

        % Print time at which we are solving in days
        disp(t);

        % If dissolution occurs ...
        if(case_id == 1 || case_id == 3) 
            
            % Perform Gauss-Seidel iteration with Type 1 (zero
            % conentration) boundary conditions:

            % The initial guess for each timestep is the value from the 
            % previous step
            num_conc(2:n_bins, t_index+1)= num_conc(2:n_bins, t_index);   

            counter = 0;
            
            % Convergence (within some error tolerance) must have occurred   
            % at all bins (not including boundary nodes) for the solution 
            % to be considered converged at the given timestep
            while counter < (n_bins-3)  
                
                % Flag for test of convergence at each time step
                counter = 0; 

                % For all bins ...
                for bin_i = 2:(n_bins-2) 

                    % Collect values calculated during the previous 
                    % iteration so we can test for convergence at the end 
                    % of the current iteration
                    conc_prev = num_conc(bin_i, t_index+1);  

                    % Calculate the ion gradient that is the driving force
                    % for dissolution
                    if(sink == 0)
                        ion_grad = S-mass_conc_ion(1,t_index);
                    else
                        ion_grad = S;
                    end
                   
                   % Dissolution only
                   if(case_id == 1)
                       % Second order finite difference from Hounslow et 
                       % al. (1988) and Kostoglou and Karabelas (2007) 
                       % Eq. S20 of Dale et al. (2016)
                        a_const = - (2*m_rat) / (  (1+m_rat)*(m_rat^2 - 1)  );
                        b_const = - 2/(1+m_rat);
                        c_const =   (2*m_rat) /  (  (1+m_rat)*(m_rat^2 - 1)  );

                        num_conc(bin_i, t_index+1)   = (max(num_conc(bin_i, t_index),0) + ...
                           (1/bin_i_lo_m(bin_i,1))*k_S_pb*ion_grad*delt*(bin_i_mid_mom(4, bin_i-1)^pow)*(  max(num_conc(bin_i-1, t_index+1),0)*a_const  ) + ... 
                           (1/bin_i_lo_m(bin_i,1))*k_S_pb*ion_grad*delt*(bin_i_mid_mom(4, bin_i+1)^pow)*(  max(num_conc(bin_i+1, t_index),0)*c_const  ))*...
                           (1/(1 - (1/bin_i_lo_m(bin_i,1))*k_S_pb*ion_grad*delt*(bin_i_mid_mom(4, bin_i)^pow)*b_const));

                        if(num_conc(bin_i, t_index+1) <= 0)
                            num_conc(bin_i, t_index+1) = 0;
                        end  
                        
                   % Simultaneous dissolution and aggregation, with or 
                   % without fusion (pass 1)
                   elseif(case_id == 3 && case3_pass == 1 )
                       % First order finite difference as described by 
                       % Kostoglou and Karabelas (2007) (performs better 
                       % for simultaneous dissolution and aggregation)
                       num_conc(bin_i, t_index+1) = (num_conc(bin_i, t_index) + ...
                          k_S_pb*ion_grad*delt*(bin_i_mid_mom(4, bin_i+1)^pow)*(num_conc(bin_i+1, t_index+1)/(bin_i_lo_m(bin_i+2,1)-bin_i_lo_m(bin_i+1,1))) ) ...
                          *(1/(1 + k_S_pb*ion_grad*delt*(bin_i_mid_mom(4, bin_i)^pow)*(1/(bin_i_lo_m(bin_i+1,1)-bin_i_lo_m(bin_i,1)))  ));
                      
                   % Simultaneous dissolution and aggregation, no fusion
                   % (pass 2)
                   elseif(case_id == 3 && case3_pass == 2)
                       % First order finite difference
                       num_conc(bin_i, t_index+1) = (num_conc(bin_i, t_index) + ...
                          k_S_pb*(bin_i_mid_mom(4, bin_i+1)/char_mass(1,t_index))*ion_grad*delt*(char_mass(1,t_index)^pow)*(num_conc(bin_i+1, t_index+1)/(bin_i_lo_m(bin_i+2,1)-bin_i_lo_m(bin_i+1,1))) ) ...
                          *(1/(1 + k_S_pb*(bin_i_mid_mom(4, bin_i)/char_mass(1,t_index))*ion_grad*delt*(char_mass(1,t_index)^pow)*(1/(bin_i_lo_m(bin_i+1,1)-bin_i_lo_m(bin_i,1)))  ));
                   end

                    % Test for convergence (break condition)
                    if(num_conc(bin_i, t_index+1) - conc_prev < 1e-50)   %error tolerance
                        counter = counter + 1;  
                    end

                end   %end bin loop 
            end   %end while  
        end


        % If aggregation occurs ...
        % Numerical solution for aggregation given by Lister et al. (1995)
        % Eq. S21 of Dale et al. (2016)
        % Bins are looped over separately for dissolution and aggregation
        % because dissolution requires an implicit solution, whereas an
        % explicit solution is preferable for aggregation
        if(case_id == 2 || (case_id == 3 && case3_pass == 2) || (case_id == 3 && fusion == 1)) 
            
            for bin_i = 2:(n_bins-1)

                delta_agg = 0;

                for bin_j = 2:(bin_i-sum(1:q)-1)
                    beta_ag = prefac_agg*( ((bin_i_mid_mom(4, bin_i-1)^(1/Df) + bin_i_mid_mom(4, bin_j)^(1/Df))^2) / ((bin_i_mid_mom(4, bin_i-1)*bin_i_mid_mom(4, bin_j))^(1/Df)) );
                    delta_agg = delta_agg + ((2^((bin_j-bin_i+1)/q))/(2^(1/q)-1))*beta_ag*max(num_conc(bin_i-1, t_index),0)*max(num_conc(bin_j, t_index),0);
                end

                for bin_k = 2:q
                    for bin_j = max(2,(bin_i-sum(1:(q-bin_k+2))-bin_k+1)):(bin_i-sum(1:(q-bin_k+1))-bin_k)
                        if((bin_i-bin_k) > 1)
                            beta_ag = prefac_agg*( ((bin_i_mid_mom(4, bin_i-bin_k)^(1/Df) + bin_i_mid_mom(4, bin_j)^(1/Df))^2) / ((bin_i_mid_mom(4, bin_i-bin_k)*bin_i_mid_mom(4, bin_j))^(1/Df)) );
                            delta_agg = delta_agg + ((2^((bin_j-bin_i+1)/q)-1+2^(-(bin_k-1)/q))/(2^(1/q)-1))*beta_ag*max(num_conc(bin_i-bin_k, t_index),0)*max(num_conc(bin_j, t_index),0);
                        end
                    end
                end

                if((bin_i-q) >= 2)
                    beta_ag = prefac_agg*( ((bin_i_mid_mom(4, bin_i-q)^(1/Df) + bin_i_mid_mom(4, bin_i-q)^(1/Df))^2) / ((bin_i_mid_mom(4, bin_i-q)*bin_i_mid_mom(4, bin_i-q))^(1/Df)) );
                    delta_agg = delta_agg + 0.5*beta_ag*(max(num_conc(bin_i-q, t_index),0)^2);
                end

                for bin_k = 2:q
                    for bin_j = max(2,(bin_i-sum(1:(q-bin_k+2))-bin_k+2)):(bin_i-sum(1:(q-bin_k+1))-bin_k+1)
                        if((bin_i-bin_k+1) > 1)
                            beta_ag = prefac_agg*( ((bin_i_mid_mom(4, bin_i-bin_k+1)^(1/Df) + bin_i_mid_mom(4, bin_j)^(1/Df))^2) / ((bin_i_mid_mom(4, bin_i-bin_k+1)*bin_i_mid_mom(4, bin_j))^(1/Df)) );
                            delta_agg = delta_agg + ((-2^((bin_j-bin_i)/q)+2^(1/q)-2^(-(bin_k-1)/q))/(2^(1/q)-1))*beta_ag*max(num_conc(bin_i-bin_k+1, t_index),0)*max(num_conc(bin_j, t_index),0);
                        end
                    end
                end

                for bin_j = 2:(bin_i-sum(1:q))      
                    beta_ag = prefac_agg*( ((bin_i_mid_mom(4, bin_i)^(1/Df) + bin_i_mid_mom(4, bin_j)^(1/Df))^2) / ((bin_i_mid_mom(4, bin_i)*bin_i_mid_mom(4, bin_j))^(1/Df)) );
                    delta_agg = delta_agg - ((2^((bin_j-bin_i)/q))/(2^(1/q)-1))*beta_ag*max(num_conc(bin_j, t_index),0)*max(num_conc(bin_i, t_index),0);
                end

                for bin_j = max(2,(bin_i-sum(1:q)+1)):(n_bins-1)
                    beta_ag = prefac_agg*( ((bin_i_mid_mom(4, bin_i)^(1/Df) + bin_i_mid_mom(4, bin_j)^(1/Df))^2) / ((bin_i_mid_mom(4, bin_i)*bin_i_mid_mom(4, bin_j))^(1/Df)) );
                    delta_agg = delta_agg - beta_ag*max(num_conc(bin_j, t_index),0)*max(num_conc(bin_i, t_index),0);
                end

                if(case_id == 2)
                    num_conc(bin_i, t_index+1) = num_conc(bin_i, t_index)  + delt*delta_agg;
                else
                    num_conc(bin_i, t_index+1) = num_conc(bin_i, t_index+1)  + delt*delta_agg;
                end

                % These three lines and the use of max(num_conc(bin_i,
                % t_index),0) above prevents numerical errors due to the
                % number of particles in a bin going negative
                if(num_conc(bin_i, t_index+1) <= 0)
                  num_conc(bin_i, t_index+1) = 0;
                end
                
            end   %end bin loop 
        end

        % Calculate changes in the ion concentration during the time step
        % due to dissolution
        if((case_id == 1 || case_id == 3) && sink == 0)
            
            % Solve for the total NP mass concentration in g/m^3 at times t
            % and t+1
            sum_t = sum(bin_i_mid_mom(4, 2:n_bins-2)'.*num_conc(2:n_bins-2, t_index));
            sum_tplus = sum(bin_i_mid_mom(4, 2:n_bins-2)'.*num_conc(2:n_bins-2, t_index+1));
            mass_conc_ion(1,t_index+1) = mass_conc_ion(1,t_index)-g_rat*(sum_tplus-sum_t);  
        end
 
        % Calculate the characteristic radius at time t, if needed
        if(case_id == 1 || (case_id == 3 && case3_pass == 1))
            
            char_rad_numer = 0;
            for bin_i=2:n_bins-2
                char_rad_numer = char_rad_numer + bin_i_mid_mom(3, bin_i)*log((6/(rho_p*pi))^(1/3)*bin_i_mid_mom(2, bin_i))*num_conc(bin_i,t_index);
            end
            
            char_rad_denom = 0;
            for bin_i=2:n_bins-2
                char_rad_denom = char_rad_denom + bin_i_mid_mom(3, bin_i)*num_conc(bin_i,t_index);
            end
            
            char_rad(1,t_index) = exp(char_rad_numer./char_rad_denom)/2;
        end

    end    %end time loop

    time = toc; % end timer


    %---------------------------------------------------------------------%
    %%%%%%%%%%%%%%%%%%%%%%%%%%% POST-PROCESSING %%%%%%%%%%%%%%%%%%%%%%%%%%%  
    %---------------------------------------------------------------------%
    
    % Calculate the raw fractional moments of the distribution
    for k = 1:num_moments
        for t_index=1:n_t
            for bin_i=2:n_bins-2
                stats(k,t_index) = stats(k,t_index) + bin_i_mid_mom(k, bin_i)*num_conc(bin_i,t_index);
            end
        end
    end

    
    % Dissolution
    if(case_id == 1 || (case_id == 3 && fusion == 1) ) 
        
        % Convert the characteristic radius in m to a diameter in nm
        stats(2,:) = 2*(10^9)*char_rad(1,:);  
        
        % Use the geometry of a sphere to convert the raw 2/3 moment to the 
        % surface area concentration in m^2/m^3
        stats(3,:) = stats(3,:)*(4*pi)*((4/3)*rho_p*pi)^(-(2/3));  
        
    % Aggregation
    elseif(case_id == 2) 
        
        % Calculate the geometric mean number of primary particles in the 
        % aggregate
        for t_index = 1:n_t
            for bin_i=2:n_bins-2
                mean_num_per(1,t_index) = mean_num_per(1,t_index) + ( log(    bin_i_mid_mom(4, bin_i)  /   char_mass0    )*num_conc(bin_i,t_index));
            end
        end
        mean_num_per(1,:) = exp(mean_num_per(1,:)./stats(1,:));
        
        % Characteristic diameter of the aggregate (in nm) as described by 
        % Eq. S10 of Dale et al. (2016)
        stats(2,:)=char_rad0*((  mean_num_per(1,:)  ).^(1/Df))*2*(10^9);
        
        % The surface area concentration (in m^2/m^3) does not change over 
        % time due to particle fusion in the absence of aggregation
        stats(3,:)=N0*char_sa0;  
                
    % Simultaneous aggregation and dissolution, no fusion
    elseif(case_id == 3 && case3_pass == 2) 

        % Calculate the geometric mean number of primary particles in the 
        % aggregate
        for t_index = 1:n_t
            for bin_i=2:n_bins-2
                mean_num_per(1,t_index) = mean_num_per(1,t_index) +  (log(    bin_i_mid_mom(4, bin_i)  ./   char_mass(1,t_index)    )*num_conc(bin_i,t_index));
            end
        end
        mean_num_per(1,:) = exp(mean_num_per(1,:)./stats(1,:));
        
        % Characteristic diameter of the aggregate (in nm) as described by
        % Eq. S10 of Dale et al. (2016)
        stats(2,:)=char_rad(1,:).*((  mean_num_per(1,:)  ).^(1/Df))*2*(10^9); 
        
        % The surface area concentration (in m^2/m^3) does not change over 
        % time due to particle fusion, only dissolution
        stats(3,:)=N0.*char_sa(1,:); 
  
    end
  
    % Replace zero values with NaN so that they are not plotted
    stats(stats == 0) = NaN;

    return

end


% KEY REFERENCES
% For a complete list of references, see Dale et al. (2016)

% Hounslow, M., R. Ryall, and V. Marshall (1988), A discretized population 
% balance for nucleation, growth, and aggregation, AIChE Journal, 
% 34(11), 1821-1832.

% Kostoglou, M., and A. Karabelas (1995), Evaluation of numerical methods 
% for simulating an evolving particle size distribution in growth 
% processes, Chemical Engineering Communications, 136(1), 177-199.

% Lister, J., D. Smit, and M. Hounslow (1995), Adjustable discretized 
% population balance for growth and aggregation, AIChE Journal, 41(3), 
% 591-603.

